# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#
from random import randint, shuffle, randrange

from pyromaths.outils.decimaux import decimaux


# from pyromaths.outils.Geometrie import cotation, cotation_h

def choix_valeurs(min_prct, max_prct, min_nbre, max_nbre):
    while 1:
        decomp100 = [2, 2, 5, 5, 1, 1]
        shuffle(decomp100)
        prct = decomp100[0] * decomp100[1] * decomp100[2]  # qui donne 2, 4, 5, 10, 20, 25 ou 50
        tot = decomp100[3] * decomp100[4] * decomp100[5]
        if (max_prct / prct != 0) and (max_nbre / tot != 0):
            break
    prct = randint(max(min_prct // prct, 1), (max_prct // prct)) * prct
    tot = tot * randint(max(min_nbre // tot, 1), (max_nbre // tot))
    return prct, tot


def proportionnalite_3eme():
    # choix exercice
    i = randrange(2)
    # choix valeurs
    # ===========================================================================
    # [min_prct, max_prct, min_nbre, max_nbre] = [[40, 80, 40, 65],
    #                                             [40, 80, 65, 90]][i]
    # ===========================================================================
    # choisit les valeurs du groupe A
    # prctA : pourcentage d'individu dans le groupe A entre 40% et 80%
    # totA : nombre total du groupe A entre 40 et 70
    prctA, totA = choix_valeurs(min_prct=40, max_prct=80, min_nbre=40, max_nbre=65)

    # On choisit B avec des valeurs différentes
    while 1:
        prctB, totB = choix_valeurs(min_prct=40, max_prct=80, min_nbre=40, max_nbre=65)
        if prctA != prctB and totB != totA:
            break

    Total = totA + totB

    # calcul
    nbA = (prctA * totA / 100)
    nbB = (prctB * totB / 100)
    nbTotal = ((nbA + nbB))
    prct_final = (100.0 * nbTotal / Total)
    prct_final_arrondi = round(100.0 * nbTotal / Total, 1)
    if prct_final == prct_final_arrondi:
        approx = "="
    else:
        approx = "\\simeq"

    # La situation
    texte = [u"Lors d'un voyage scolaire, les élèves sont répartis dans deux bus :",
             u"Au collège Dubois, il y a $%s$ élèves en quatrièmes et $%s$ élèves en troisièmes." % (totA, totB)][i]

    # Les deux groupes
    ligne1 = [u"\\item Bus A : %s élèves dont %s %s de garçons." % (totA, prctA, "\\%"),
              u"\\item $%s$ %s des élèves de quatrièmes possèdent un ordinateur ;" % (prctA, "\\%")][i]
    ligne2 = [u"\\item Bus A : %s élèves dont %s %s de garçons." % (totB, prctB, "\\%"),
              u"\\item $%s$ %s des élèves de troisièmes possèdent un ordinateur ;" % (prctB, "\\%")][i]

    # La question
    question = [u"Quel est, sur l'ensemble des deux bus, le pourcentage de garçons ?\\par",
                u"Quel est le pourcentage des élèves qui possèdent un ordinateur ?\\par"][i]

    # La correction
    correction = [  # Dans le bus
        [u"Dans le bus A, il y a $\\cfrac{%s \\times %s}{100} = %s$ garçons.\\par" % (prctA, totA, decimaux(nbA)),
         u"Dans le bus B, il y a $\\cfrac{%s \\times %s}{100} = %s$ garçons.\\par" % (prctB, totB, decimaux(nbB)),
         u"On en déduit qu'il y a $%s + %s = %s$ garçons sur un total de $%s + %s = %s $ élèves.\\par"
         % (decimaux(nbA), decimaux(nbB), nbTotal, totA, totB, Total),
         u" Le pourcentage de garçons sur l'ensemble des deux bus est donc de $\\cfrac{%s}{%s}\\times 100 %s %s%s$"
         % (nbTotal, Total, approx, decimaux(prct_final_arrondi), "\,\\%")],
        # À l'ordinateurs
        [u"En quatrièmes, $\\cfrac{%s \\times %s}{100} = %s$ élèves possèdent un ordinateur.\\par" % (
        prctA, totA, decimaux(nbA)),
         u"En troisièmes, $\\cfrac{%s \\times %s}{100} = %s$ élèves possèdent un ordinateur.\\par" % (
         prctB, totB, decimaux(nbB)),
         u"On en déduit qu'il y a $%s + %s = %s$ élèves qui possèdent un ordinateur sur un total de $%s + %s = %s $ élèves.\\par"
         % (decimaux(nbA), decimaux(nbB), nbTotal, totA, totB, Total),
         u" Le pourcentage d'élèves possédant un ordinateur est donc de $\\cfrac{%s}{%s}\\times 100 %s %s%s$"
         % (nbTotal, Total, approx, decimaux(prct_final_arrondi), "\,\\%")],
    ][i]
    exo = ["\\exercice",
           texte,
           "\\begin{itemize}",
           ligne1,
           ligne2,
           "\\end{itemize}",
           question,
           ]

    cor = ["\\exercice*",
           texte,
           "\\begin{itemize}",
           ligne1,
           ligne2,
           "\\end{itemize}",
           question,
           "\\dotfill\\par"
           ] + correction

    return (exo, cor)



#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

import codecs
import contextlib
import functools
import logging
import os
import subprocess
import shutil
import sys
import tempfile
import textwrap

from pyromaths.outils import jinja2tex
from pyromaths import directories

#==============================================================
#        Gestion des extensions de fichiers
#==============================================================
def supprime_extension(filename, ext):
    """Supprime l'éventuelle extension ext du nom de fichier filename.

    - ext est une chaîne de caractères quelconque.

    >>> supprime_extension("plop.tex", ".tex")
    'plop'
    >>> supprime_extension("plop.tex", ".pdf")
    'plop.tex'
    """
    if filename.endswith(ext):
        return filename[:-len(ext)]
    return filename

################################################################################

class Fiche:
    basename = "exercise"

    def __init__(self, context, *, template="pyromaths.tex", dirty=False):
        self.context = context
        self.template = template
        self.dirty = dirty

    def __enter__(self):
        self.workingdir = tempfile.mkdtemp(prefix="pyromaths-")
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        if self.dirty:
            logging.info("Temporary directory '%s' not removed (as requested).", self.workingdir)
        else:
            shutil.rmtree(self.workingdir)

    def tempfile(self, ext=None):
        if ext is None:
            return os.path.join(self.workingdir, self.basename)
        return os.path.join(self.workingdir, "{}.{}".format(self.basename, ext))

    @property
    def texname(self):
        return self.tempfile("tex")

    @property
    def pdfname(self):
        return self.tempfile("pdf")

    @property
    def latexmkrcname(self):
        return os.path.join(self.workingdir, "latexmkrc")

    @functools.lru_cache(10)
    def write_tex(self):
        environment = jinja2tex.LatexEnvironment(
            loader=jinja2tex.FileSystemLoader([
                os.path.join(directories.CONFIGDIR, 'templates'),
                os.path.join(directories.DATADIR, 'templates'),
                ])
        )
        with codecs.open(self.texname, mode='w', encoding='utf-8') as exofile:
            exofile.write(environment.get_template(self.template).render(self.context))

    def write_pdf(self):
        self.write_tex()
        self.write_latexmkrc()
        if os.name == 'nt':
            subprocess.run(
                ["latexmk", "-norc", "-r", "latexmkrc", self.basename],
                cwd=self.workingdir,
                env={"PATH": os.environ['PATH'], "WINDIR": os.environ['WINDIR'], 'USERPROFILE': os.environ['USERPROFILE']},
                )
            subprocess.run(
                ["latexmk", "-norc", "-r", "latexmkrc", "-c"],
                cwd=self.workingdir,
                env={"PATH": os.environ['PATH'], "WINDIR": os.environ['WINDIR'], 'USERPROFILE': os.environ['USERPROFILE']},
                )
        else:
            subprocess.run(
                ["latexmk", "-norc", "-r", "latexmkrc", self.basename],
                cwd=self.workingdir,
                )
            subprocess.run(
                ["latexmk", "-norc", "-r", "latexmkrc", "-c"],
                cwd=self.workingdir,
                )

    @functools.lru_cache(1)
    def write_latexmkrc(self):
        with open(self.latexmkrcname, 'w') as latexmkrc:
            latexmkrc.write(textwrap.dedent("""\
            $pdf_mode = 2;
            $ps2pdf = "ps2pdf %O %S %D";
            $latex = "latex --shell-escape -interaction=nonstopmode  %O %S";
            $auto_rc_use=0;
            $silent = 1;
            sub asy {return system("asy '$_[0]'");}
            add_cus_dep("asy","eps",0,"asy");
            add_cus_dep("asy","pdf",0,"asy");
            add_cus_dep("asy","tex",0,"asy");
            $cleanup_mode = 2;
            $clean_ext .= " %R-?.tex %R-??.tex %R-figure*.dpth %R-figure*.dvi %R-figure*.eps %R-figure*.log %R-figure*.md5 %R-figure*.pre %R-figure*.ps %R-figure*.asy %R-*.asy %R-*_0.eps %R-*.pre";
            """))
            #latexmkrc.write('push @generated_exts, \'pre\', \'dvi\', \'ps\', \'auxlock\', \'fdb_latexmk\', \'fls\', \'out\', \'aux\';\n')
            #latexmkrc.write('@generated_exts = qw(4ct 4tc acn acr alg aux auxlock bbl dvi eps fls glg glo gls idv idx ind ist lg lof lot nav net out pre ps ptc run.xml snm thm tmp toc vrb xdv xref);')


    def show_pdf(self, filename=None):
        if filename is None:
            filename = self.pdfname
        if os.name == "nt":  # Cas de Windows.
            os.startfile(filename)
        elif sys.platform == "darwin":  # Cas de Mac OS X.
            subprocess.run(['open', filename])
        else:
            subprocess.run(['gio', 'open', filename])




from random import randint, randrange

from pyromaths.classes.Polynome import Polynome
from .Arithmetique import pgcd


# from math import *


# --------------outils pour la class Polynome----------------------

def TeX_division(dividende, diviseur):
    '''renvoie une chaine de caractere TeX pour afficher la division en détail'''
    #     quotient = Polynome({0:0})
    texquotient, restera = dividende / diviseur
    reste = dividende
    diviseur_degre = diviseur.deg
    sauve = min(dividende.puiss + restera.puiss)
    longueur = dividende.degre_max - sauve
    string = "$$\\renewcommand\\arraycolsep{0cm}\n\\begin{array}{c"
    for dummy in range(longueur):
        string += "c"
    string += "|c}\n"
    string += tab_print(dividende, longueur + 1)
    string += str(diviseur) + "\\\\\n"
    string += "\\cline{" + str(dividende.deg + 2 - sauve) + "-" + str(dividende.deg + 2 - sauve) + "}\n"
    i = 0
    while reste.deg >= diviseur.deg:
        for dummy in range(i):
            string += " &"
        ajout_quotient_deg = reste.deg - diviseur_degre
        facteur = reste.dictio[max(0, reste.deg)] / diviseur.dictio[diviseur.deg]
        ajout_quotient = Polynome({ajout_quotient_deg: facteur}, var=dividende.var)
        soustrait_reste = ajout_quotient * diviseur
        string += tab_print(soustrait_reste, diviseur_degre + 1 - sauve, parenthese=True)
        for dummy in range(longueur - i - diviseur.deg + sauve):
            string += " & "
        if i == 0:
            string += str(texquotient)
        string += "\\\\\n"
        string += "\\cline{" + str(i + 1) + "-" + str(i + diviseur.deg + 1 - sauve) + "}\n"
        avant = reste.deg
        reste = reste - soustrait_reste
        delta = avant - reste.degre_max
        for dummy in range(i):
            string += " &"
        i = i + delta
        #         prochain = 1
        string += tab_print(reste, min(diviseur.deg + 1, reste.degre_max + 1) + delta - sauve, debut=delta)
        # fait descendre les monome du dividende
        for dummy in range(longueur - i - diviseur.deg + sauve):
            string += " & "
        string += "\\\\ \n"
    string += "\\end{array}\n$$"

    string += "\n On a $$" + dividende.TeX() + " = \\left(" + texquotient.TeX() + "\\right) \\times \\left(" + diviseur.TeX() + "\\right)"
    if restera != Polynome(0):
        if len(restera.puiss) == 1 and restera[restera.deg] > 0:  # monome
            string += "+" + restera.TeX()
        else:
            string += "+\\left(" + restera.TeX() + "\\right)"
    string += "$$"
    return string


def tab_print(polynome, longueur=0, parenthese=False, debut=0):
    '''utilisé par TeX_division pour décaler le reste dans la partie gauche'''
    degre = polynome.degre_max + debut
    string = ''
    if parenthese:
        string = "-("
        fin = ")"
    else:
        fin = ""
    if polynome == 0:
        string += "+" + str(0) + " &"
    else:
        for i in range(longueur):
            k = degre - i
            coeff = polynome.dictio.get(k, 0)
            if coeff >= 0:
                string += "+"
            string += str(coeff)
            if k != 0:
                string += polynome.var
            if k != 1 and k != 0:
                string += u"^" + str(k)
            if longueur - i == 1:
                string += fin
            string += " & "
    return string


########################################################
#
#  construction de polynôme de degré 2
#  X est un Polynome
#  X=Polynome("x","x") donnera un polynome de degré 2
#  X=Polynome("x^2",x) donnera un polynôme bicarrée
#
########################################################
def poly_racines_quelconques(abs_a, abs_b, abs_c, X):
    '''renvoie un polynome de degré 2'''
    '''abs_a,abs_b,abs_c sont des entiers positifs majorant les valeurs absolues de a, b ,c'''

    a3 = (2 * randrange(2) - 1) * randrange(1, abs_a + 1)
    b3 = randrange(abs_b)
    c3 = randrange(-abs_c, abs_c)
    return a3 * X ** 2 + b3 * X + c3


def poly_racines_fractionnaires(rac_min, rac_max, denom1, X):
    '''renvoie un polynome de degré2 à racines fractionnaires '''
    '''les racines sont comprises entre rac_min et rac_max'''
    '''denom1 majore le dénominateur des racines'''
    while 1:
        # pour éviter P=77x^2
        a2 = 2 * randrange(2) - 1  # a2=-1 ou 1
        p2facteur = [randint(1, denom1) * X - randint(rac_min, rac_max) for dummy in range(2)]
        pol2 = a2 * p2facteur[0] * p2facteur[1]
        if pol2[1] != 0 and pol2[0] != 0:
            break
    pol2 = pol2.simplifie()
    simplifie = abs(pgcd(pgcd(int(pol2[0]), int(pol2[1])), int(pol2[2])))
    pol2 = pol2 / simplifie
    return pol2.simplifie()


def poly_racines_entieres(rac_min, rac_max, X, a1=1):
    while 1:
        p1facteur = [X - randrange(rac_min, rac_max) for dummy in range(2)]
        pol1 = a1 * p1facteur[0] * p1facteur[1]
        if pol1[1] != 0 or pol1[1] != 0:
            break
    return pol1


def poly_id_remarquables(rac_min, rac_max, X):
    '''Renvoie un polynome obtenu par une identité remarquable'''
    '''et sgns 
         -2 => (a-b)²
         +2 => (a+b)²
          0 => (a-b)(a+b)'''
    a = randint(1, 10)
    while 1:
        coeff = randrange(rac_min, rac_max)
        racine = randrange(rac_min, rac_max)
        if coeff != 0 and racine != 0:
            break
    sgns = [[-1, 1][randrange(2)] for dummy in range(2)]
    p1facteur = [coeff * X + sgns[i] * racine for i in range(2)]
    return a * p1facteur[0] * p1facteur[1], sum(sgns)
    # sum(sgns) permet de connaître l'identité remarquable
    # -2 => (a-b)²
    # +2 => (a+b)²
    # 0 => (a-b)(a+b)


# ----------polynome de degré 3-------------
def poly_degre3_racines_entieres(rac_min, rac_max, X, racines=[-2, -1, 0, 1, 2]):
    racine_evidente = racines[randrange(len(racines))]
    return (X - racine_evidente) * poly_racines_entieres(rac_min, rac_max, X)


def poly_degre3_racines_fractionnaires(rac_min, rac_max, denom1, X, racines=[-2, -1, 0, 1, 2]):
    racine_evidente = racines[randrange(len(racines))]
    return (X - racine_evidente) * poly_racines_fractionnaires(rac_min, rac_max, denom1, X)


def poly_degre3_racines_quelconques(abs_a, abs_b, abs_c, X, racines=[-2, -1, 0, 1, 2]):
    racine_evidente = racines[randrange(len(racines))]
    return (X - racine_evidente) * poly_racines_quelconques(abs_a=1, abs_b=10, abs_c=10, X=X)



#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#
import random
from math import cos, sin, radians, pi
# from pyromaths.outils.decimaux import decimaux
from pyromaths.outils.Arithmetique import pgcd
from pyromaths.ex import LegacyExercise

div180 = [[1,180],[2,90],[3,60],[4,45],[5,36],[6,30],[9,20],[10,18],[12,15]]

# Radians représentés en fait par des fractions

def deg2rad(n):
    """Effectue la conversion de degrés entre 0 et 360° vers radians."""
    p = pgcd(n,180)
    return [n // p,180 // p]

def simprad(liste):
    """Simplifie la fraction d'un angle en radians. Le paramètre est une liste d'entiers, pour représenter une fraction."""
    p = pgcd(liste[0],liste[1])
    return [f // p for f in liste]

def rad2deg(liste):
    """Effectue la conversion de radians entre 0 et 2pi vers degrés. Le paramètre est une liste d'entiers, pour représenter une fraction."""
    return liste[0]*180/liste[1] # Attention résultat entier, donc faux si liste[1] ne divise pas 180

def rad2tex(liste):
    """Prépare l'écriture de radians en latex."""
    if liste[0]==0:
        frac =  "$0$"
    elif liste[0]==liste[1]:
        frac = "$\\pi$"
    elif liste[0]==-liste[1]:
        frac = "$-\\pi$"
    elif liste[0]==1:
        frac = "$\\dfrac{\\pi}{"+str(liste[1])+"}$"
    elif liste[0]==-1:
        frac = "$\\dfrac{-\\pi}{"+str(liste[1])+"}$"
    elif liste[1]==-1: # Probablement inutile
        frac = "$"+str(-liste[0])+"\\pi$"
    elif liste[1]==1:
        frac = "$"+str(liste[0])+"\\pi$"
    else:
        frac = "$\\dfrac{"+str(liste[0])+"\\pi}{"+str(liste[1])+"}$"
    return frac

def mes_princ(liste):
    """Calcule la mesure principale d'un angle en radians ( entre -pi et pi )."""
    num = liste[0] % (2*liste[1])
    if num > liste[1]:
        num -= 2*liste[1]
    return simprad([num,liste[1]])

def mes_02pi(liste):
    """Calcule la mesure d'un angle en radians dans l'intervalle [0,2pi]."""
    num = liste[0] % (2*liste[1])
    return [num,liste[1]]

def cercle_trigo(code=0,*args):
    """Dessine le cercle trigonométrique avec un nombre indéfini d'angles en degrés ou radians, en codant ou non les angles."""
    figure = """\\psset{xunit=3cm,yunit=3cm,dotstyle=+}
    \\begin{pspicture}(-1.4,-1.4)(1.4,1.4)
    \\psframe(-1.4,-1.4)(1.4,1.4)
    \\psaxes[linewidth=0.5pt,Dx=2,Dy=2]{->}(0,0)(-1.1,-1.1)(1.1,1.1)
    \\pscircle[linecolor=gray](0,0){3}
    \\definecolor{grisclair}{gray}{0.20}
    \\colorlet{bleuclair}{blue!40!white}\n"""
    idx = 0
    if not code:
        for a in range(11):
            if (a+1) / 6 not in [0.5,1,1.5]:
                figure += "\\psline[linecolor=LightSkyBlue,linestyle=dashed](0,0)("+str(round(cos((a+1)*pi/6),2))+","+str(round(sin((a+1)*pi/6),2))+")\n"
        for a in range(9):
            if (a+1) / 5 not in [0.5,1,1.5]:
                figure += "\\psline[linecolor=LightSlateGray,linestyle=dashed](0,0)("+str(round(cos((a+1)*pi/5),2))+","+str(round(sin((a+1)*pi/5),2))+")\n"
        for a in range(4):
            figure += "\\psline[linecolor=LightGreen,linestyle=dashed](0,0)("+str(round(cos((2*a+1)*pi/4),2))+","+str(round(sin((2*a+1)*pi/4),2))+")\n"
    for f in args:
        if isinstance(f,list): # mesure en radians
            rad = simprad(f)
        else: # mesure en degrés
            rad = simprad(deg2rad(f))
        f = rad[0]*pi/rad[1] # Le dénominateur ne devrait pas être nul...
        frac = rad2tex(rad)

        figure += "\\psline[linecolor=red,linestyle=dashed](0,0)("+str(round(cos(f),2))+","+str(round(sin(f),2))+")\n"
        if code:
            figure += "\\rput("+str(round(1.25*cos(f),2))+","+str(round(1.25*sin(f),2))+"){"+frac+"}\n"
        else:
            figure += "\\rput("+str(round(1.25*cos(f),2))+","+str(round(1.25*sin(f),2))+"){$M_"+str(idx)+"$}\n"
            idx += 1
    figure += r"""\uput[dl](0,0){$O$}
    \uput[dr](1,0){$I$}
    \uput[ul](0,1){$J$}"""
    figure += "\n\\end{pspicture}"
    return figure

def findrad(itv=1, neg=0, denom_simple=1):
    """Génère un angle en radians sous certaines conditions :
    itv ( intervalle ): 0 ( quelconque ), 1 ( mesure principale ), 2 ( entre 0 et 2 pi )
    neg : 0 ( positif ), 1 ( négatif )
    denom_simple : 0 ( quelconque ), 1 ( simple, càd 1, 2, 3, 4, 5, ou 6 ), 2 ( diviseur de 180 )"""

    global div180

    if denom_simple==1:
        denom = random.randint(2,6)
    elif denom_simple == 0:
        denom = random.randint(2,30)
    else:
        random.shuffle(div180)
        random.shuffle(div180[0])
        denom = div180[0][0]

    if itv == 1:
        sgn = (-1)**(random.randint(1,2))
        num = sgn * random.randint(1,denom) # pas grave si -pi sort
    elif itv == 2:
        num = random.randint(1,2*denom)
    else:
        numliste = [random.randint(2*denom, 120),random.randint(-2*denom, -denom)]
        random.shuffle(numliste)
        num = numliste[0]

    if (neg and (num > 0)) or ((not neg) and (num < 0)):
        num = -num

    return [num,denom] # FIX Simplifier ?

def _cercle_trigonometrique():
    """Exercice de la fiche."""

    exo = ["\\exercice"]
    cor = ["\\exercice*"]

    mes_deg = [] # Question 1
    while len(mes_deg)<5:
        tmp = random.randint(0,360) # FIX trop complexes parfois
        if tmp not in mes_deg:
            mes_deg.append(tmp)
    mes_deg_tex = [_(u"$%s\\degres$") %(d) for d in mes_deg]

    radq2 = [] # Question 2
    while len(radq2)<5:
        tmp = findrad(2,0,2) # Dénominateur est un diviseur de 180
        if tmp not in radq2:
            radq2.append(tmp)

    radq3 = [] # Question 3
    while len(radq3)<5:
        while len(radq3)<4:
            tmp = findrad(0,0,0)
            if tmp not in radq3:
                radq3.append(tmp)
        tmp = findrad(0,1,0) # dernier négatif
        if tmp not in radq3:
            radq3.append(tmp)

    radq4 = [] # Question 4
    while len(radq4)<4:
        tmp = mes_princ(findrad(0,0,1))
        if tmp not in radq4:
            radq4.append(tmp)

    radq5 = [] # Question 5
    radq5_princ = []
    while len(radq5)<4:
        while len(radq5)<2:
            tmp = findrad(1,0,1)
            tmp_princ = mes_princ(tmp)
            if tmp_princ not in radq5_princ:
                radq5.append(tmp)
                radq5_princ.append(tmp_princ)
        while len(radq5)<3:
            tmp = findrad(1,1,1) # avant-dernier négatif
            tmp_princ = mes_princ(tmp)
            if tmp_princ not in radq5_princ:
                radq5.append(tmp)
                radq5_princ.append(tmp_princ)
        tmp = findrad(0,0,1) # dernier non principal
        tmp_princ = mes_princ(tmp)
        if tmp_princ not in radq5_princ:
            radq5.append(tmp)

    mes_rad = radq2 + radq3 + radq4 + radq5
    mes_rad_tex = [rad2tex(r)for r in mes_rad]

    mes_deg_en_rad = [deg2rad(d) for d in mes_deg]
    mes_deg_en_rad_tex = [rad2tex(r) for r in mes_deg_en_rad]

    exo.append(u"\\begin{enumerate}")
    exo.append(_(u"\\item Convertir les cinq mesures suivantes en radians : %s, %s, %s, %s et %s.") %(tuple(mes_deg_tex)))
    exo.append(_(u"\\item Convertir les cinq mesures suivantes en degrés : %s, %s, %s, %s et %s~rad.") %(tuple(mes_rad_tex[0:5])))
    exo.append(_(u"\\item Déterminer les mesures principales des angles suivants en radians : %s, %s, %s, %s et %s~rad.") %(tuple(mes_rad_tex[5:10])))
    exo.append(_(u"\\item Des angles ont été placés sur le cercle trigonométrique ci-dessous, représentés en rouge par les points $M_0$, $M_1$, $M_2$ et $M_3$. Lire leurs mesures principales en radians"))
    exo.append(_(u"( les lignes vertes, grises et bleues représentent des angles multiples de $\\dfrac{\\pi}{3}$, de $\\dfrac{\\pi}{4}$ et de $\\dfrac{\\pi}{5}$ ).\\par"))
    exo.append(cercle_trigo(0,radq4[0],radq4[1],radq4[2],radq4[3]))
    exo.append(_(u"\\item Placer les angles suivants sur le cercle trigonométrique : %s, %s, %s et %s~rad.\\par") %(tuple(mes_rad_tex[14:])))
    exo.append(cercle_trigo())
    exo.append(u"\\end{enumerate}\\par")

    cor.append(u"\\begin{enumerate}")
    cor.append(_(u"\\item Convertir les cinq mesures suivantes en radians : %s, %s, %s, %s et %s.\\par") %(tuple(mes_deg_tex)))

    cor.append(_(u"La conversion est en fait une simple règle de proportionnalité : il faut multiplier par $\\dfrac{\\pi}{180}$.\\par"))
    cor.append(_(u"Par exemple pour la première mesure, on obtient avec simplification : $"+str(mes_deg[0])+"\\times\\dfrac{\\pi}{180}$ = ")+mes_deg_en_rad_tex[0]+"~rad.\\par")
    cor.append(_(u"De même pour les autres mesures, on trouve alors respectivement : %s~rad, %s~rad, %s~rad, %s~rad et %s~rad.") %(tuple(mes_deg_en_rad_tex)))

    cor.append(_(u"\\item Convertir les cinq mesures suivantes en degrés : %s, %s, %s, %s et %s~rad.\\par") %(tuple(mes_rad_tex[0:5])))

    cor.append(_(u"On effectue alors la proportionnalité inverse : il faut multiplier par $\\dfrac{180}{\\pi}$.\\par"))
    cor.append(_(u"Après simplification, voici les résultats : %s\\degres, %s\\degres, %s\\degres, %s\\degres et %s\\degres.") %(tuple([rad2deg(r) for r in radq2])))

    cor.append(_(u"\\item Déterminer les mesures principales des angles suivants en radians : %s, %s, %s, %s et %s~rad.\\par") %(tuple(mes_rad_tex[5:10])))

    rep = mes_princ(radq3[0])
    nb_tours = (radq3[0][0]-rep[0]) // (2*radq3[0][1])

    cor.append(_(u"Une mesure d'angle en radians est définie modulo $2\\pi$, c'est-à-dire que l'ajout ou la suppression d'un tour ( qui vaut $2\\pi$ ou 360\\degres ) ne change pas un angle.\\par"))
    cor.append(_(u"Concrètement, avec le premier angle de la question, on remarque que :\\par"))
    cor.append(rad2tex(radq3[0])[:-1]+"\\equiv"+rad2tex(rep)[1:-1]+"+"+rad2tex([2*nb_tours*rep[1],rep[1]])[1:-1]+"\\equiv"+rad2tex(rep)[1:-1]+"+"+rad2tex([2*nb_tours,1])[1:-1]+"\\equiv"+rad2tex(rep)[1:-1]+u"~(2\\pi)$.\\par")
    cor.append(_(u"De même pour les autres mesures, on trouve alors respectivement : %s~rad, %s~rad, %s~rad, %s~rad et %s~rad.") %(tuple([rad2tex(mes_princ(r)) for r in radq3])))

    cor.append(_(u"\\item Des angles ont été placés sur le cercle trigonométrique ci-dessous, représentés en rouge par les points $M_0$, $M_1$, $M_2$ et $M_3$. Lire leurs mesures principales en radians"))
    cor.append(_(u"( les lignes vertes, grises et bleues représentent des angles multiples de $\\dfrac{\\pi}{3}$, de $\\dfrac{\\pi}{4}$ et de $\\dfrac{\\pi}{5}$ ).\\par"))

    cor.append(_(u"Les réponses sont directement données sur le cercle trigonométrique ci-dessous :\\par"))
    cor.append(cercle_trigo(1,mes_princ(radq4[0]),mes_princ(radq4[1]),mes_princ(radq4[2]),mes_princ(radq4[3])))
    cor.append(u"\\par")
    cor.append(_(u"Les points $M_0$, $M_1$, $M_2$ et $M_3$ définissent alors respectivement les angles %s, %s, %s et %s~rad.") %(tuple(mes_rad_tex[10:14])))
    cor.append(_(u"\\item Placer les angles suivants sur le cercle trigonométrique : %s, %s, %s et %s~rad.\\par") %(tuple(mes_rad_tex[14:])))

    cor.append(_(u"Les réponses sont directement données sur le cercle trigonométrique ci-dessous :\\par"))
    cor.append(cercle_trigo(1,radq5[0],radq5[1],radq5[2],radq5[3]))
    cor.append(u"\\par")
    cor.append(_(u"Ajoutons une simple remarque pour la dernière mesure, qui n'est pas principale : il faut effectuer en premier lieu une simplification, comme à la question 3. On obtient alors :\\par"))
    cor.append(rad2tex(radq5[3])[:-1]+"\\equiv"+rad2tex(mes_princ(radq5[3]))[1:-1]+"~(2\\pi)$.")
    cor.append(u"\\end{enumerate}\\par")

    return exo, cor

class cercle_trigonometrique(LegacyExercise):
    """Cercle trigonométrique"""

    tags = ["Seconde"]
    function = _cercle_trigonometrique



#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#
'''
Créé le 9 déc. 2013

.. sectionauthor:: Jérôme Ortais <jerome.ortais@pyromaths.org>
'''
from random import shuffle, randrange

from pyromaths.classes.PolynomesCollege import Lagrange, choix_points
from pyromaths.ex import LegacyExercise

if __name__ == '__main__':
    pass

def creer_fonction(degre):
    """Crée une fonction polynôme de degré donné

    >>> notion_de_fonction.creer_fonction(2) # doctest: +SKIP
    'Polynome([[7, 2], [-1, 1], [-3, 0]], "x", details=0)'
    """
    p = []
    for exp in range(degre + 1):
        p.insert(0, [randrange(1, 10) * (-1) ** (randrange(3)), exp])
    return 'Polynome(%s, "x", details=0)' % p


def corrige(nom, fct, ant):
    from pyromaths.outils import Priorites3
    sol = []
    calc = fct(ant)
    res = Priorites3.priorites(calc)
    res = Priorites3.texify(res)
    sol.append(r"\par $%s\,(%s)=%s$\par" % (nom, ant, Priorites3.texify([Priorites3.splitting(calc)])[0]))
    sol.append('\\par\n'.join(['$%s\\,(%s)=%s$' % (nom, ant, res[j]) for j in range(len(res) - 1)]))
    sol.append(r'\par')
    sol.append('\\fbox{$%s\\,(%s)=%s$}\\\\\n' % (nom, ant, res[-1]))
    return sol

def _notion_fonction():
    """Créé un exercice bilan sur la notion de fonction"""
    from pyromaths.classes.PolynomesCollege import Polynome
    from pyromaths.outils import Priorites3
    fct = [eval(creer_fonction(1)), eval(creer_fonction(2))]
    ant = [randrange(1, 6) * (-1) ** fct[0].degre(), randrange(1, 6) * (-1) ** fct[1].degre(), randrange(1, 6) * (-1) ** fct[1].degre(), randrange(1, 6) * (-1) ** fct[0].degre()]
    shuffle(fct)
    val_exo2 = choix_points(nb=7)
    exo = [r"\exercice", r"\begin{multicols}{2}", r"\begin{enumerate}", u"\\item On donne"]
    exo.append(r"$\begin{array}[t]{l}")
    exo.append(u"f:~x \\longmapsto %s \\\\ g:~x \\longmapsto %s" % (fct[0], fct[1]))
    exo.extend([r"\end{array}$", r"\begin{enumerate}", ])
    exo.append(u"\\item Quelle est l'image de $%s$ par la fonction $f$ ?" % ant[0])
    cor = [r"\exercice", r"\begin{multicols}{2}", r"\begin{enumerate}", u"\\item On donne"]
    cor.append(r"$\begin{array}[t]{l}")
    cor.append(u"f:~x \\longmapsto %s \\\\ g:~x \\longmapsto %s" % (fct[0], fct[1]))
    cor.extend([r"\end{array}$", r"\begin{enumerate}", ])
    cor.append(u"\\item Quelle est l'image de $%s$ par la fonction $f$ ?" % ant[0])
    cor.extend(corrige('f', fct[0], ant[0]))
    exo.append(u"\\item Quelle est l'image de $%s$ par la fonction $g$ ?" % ant[1])
    cor.append(u"\\item Quelle est l'image de $%s$ par la fonction $g$ ?" % ant[1])
    cor.extend(corrige('g', fct[1], ant[1]))
    exo.append(u"\\item Calculer $f\\,(%s)$." % ant[2])
    cor.append(u"\\item Calculer $f\\,(%s)$." % ant[2])
    cor.extend(corrige('f', fct[0], ant[2]))
    exo.append(u"\\item Calculer $g\\,(%s)$." % ant[3])
    cor.append(u"\\item Calculer $g\\,(%s)$." % ant[3])
    cor.extend(corrige('g', fct[1], ant[3]))
    exo.append(r"\end{enumerate}")
    cor.append(r"\end{enumerate}")
    exo.append(u"\\item Voici un tableau de valeurs correspondant à une fonction $h$.\\par")
    exo.append(r"\renewcommand{\arraystretch}{1.5}")
    exo.append(r"\begin{tabularx}{\linewidth}[t]{|c|*7{>{\centering}X|}}")
    exo.append(r"\hline")
    exo.append(r"$x$ & %s \tabularnewline \hline" % " & ".join(["$%s$" % c[0] for c in val_exo2]))
    exo.append(r"$h\,(x)$ & %s \tabularnewline \hline" % " & ".join(["$%s$" % c[1] for c in val_exo2]))
    exo.append(r"\end{tabularx} \medskip")
    exo.append(r"\begin{enumerate}")
    cor.append(u"\\item Voici un tableau de valeurs correspondant à une fonction $h$.\\par")
    cor.append(r"\renewcommand{\arraystretch}{1.5}")
    cor.append(r"\begin{tabularx}{\linewidth}[t]{|c|*7{>{\centering}X|}}")
    cor.append(r"\hline")
    cor.append(r"$x$ & %s \tabularnewline \hline" % " & ".join(["$%s$" % c[0] for c in val_exo2]))
    cor.append(r"$h\,(x)$ & %s \tabularnewline \hline" % " & ".join(["$%s$" % c[1] for c in val_exo2]))
    cor.append(r"\end{tabularx} \medskip")
    cor.append(r"\begin{enumerate}")
    lpos21 = [i for i in range(7)]
    for dummy in range(3):
        del lpos21[randrange(len(lpos21))]
    shuffle(lpos21)
    lquest = [u"Quelle est l'image de $%s$ par la fonction $h$ ?" % val_exo2[lpos21[0]][0],
              u"Quel est l'antécédent de $%s$ par la fonction $h$ ?" % val_exo2[lpos21[1]][1],
              u"Compléter : $h\\,(%s)=\\ldots\\ldots$" % val_exo2[lpos21[2]][0],
              u"Compléter : $h\\,(\\ldots\\ldots)=%s$" % val_exo2[lpos21[3]][1]]
    lpos22 = [0, 1, 2, 3]
    shuffle(lpos22)
    for i in range(4):
        exo.append(u"\\item %s" % lquest[lpos22[i]])
        if lpos22[i] == 0:
            cor.append(u"\\item L'image de $%s$ par la fonction $h$ est $\\mathbf{%s}$." % (val_exo2[lpos21[0]][0], val_exo2[lpos21[0]][1]))
        elif lpos22[i] == 1:
            cor.append(u"\\item Un antécédent de $%s$ par la fonction $h$ est $\\mathbf{%s}$." % (val_exo2[lpos21[1]][1], val_exo2[lpos21[1]][0]))
        if lpos22[i] == 2:
            cor.append(u"\\item $h\\,(%s)=\\mathbf{%s}$." % (val_exo2[lpos21[2]][0], val_exo2[lpos21[2]][1]))
        elif lpos22[i] == 3:
            cor.append(u"\\item $h\\,(\\mathbf{%s})=%s$." % (val_exo2[lpos21[3]][0], val_exo2[lpos21[3]][1]))

    exo.extend([r"\end{enumerate}", r"\columnbreak"])
    cor.append(r"\end{enumerate}")
    val_exo3 = choix_points()
    p = Priorites3.plotify(Priorites3.priorites(Lagrange(val_exo3))[-1])
    exo.extend([u"\\item Le graphique ci-dessous représente une fonction $k$ : \\par", r"\begin{center}",
                r"\psset{unit=8mm, algebraic, dotsize=4pt 4}", r"\begin{pspicture*}(-4.2,-4.2)(4.2,4.2)"])
    exo.append(r"\psgrid[subgriddiv=2, gridwidth=.6pt,subgridcolor=lightgray, gridlabels=0pt]")
    exo.append(r"\psaxes[linewidth=1.2pt,]{->}(0,0)(-4.2,-4.2)(4.2,4.2)")
    exo.append(r"\psplot[plotpoints=200, linewidth=1.5pt, linecolor=DarkRed]{-4.2}{4.2}{%s}" % p)
    exo.append(r"\psdots %s" % " ".join([str(val) for val in val_exo3]))
    exo.extend([r"\end{pspicture*}", r"\end{center}", r"\begin{enumerate}"])
    cor.extend([u"\\item Le graphique ci-après représente une fonction $k$ : \\par", r"\begin{center}",
                r"\psset{unit=8mm, algebraic, dotsize=4pt 4}", r"\begin{pspicture*}(-4.2,-4.2)(4.2,4.2)"])
    cor.append(r"\psgrid[subgriddiv=2, gridwidth=.6pt,subgridcolor=lightgray, gridlabels=0pt]")
    cor.append(r"\psaxes[linewidth=1.2pt,]{->}(0,0)(-4.2,-4.2)(4.2,4.2)")
    cor.append(r"\psplot[plotpoints=200, linewidth=1.5pt, linecolor=DarkRed]{-4.2}{4.2}{%s}" % p)
    cor.append(r"\psdots %s" % " ".join([str(val) for val in val_exo3]))
    lpos31 = [i for i in range(5)]
    for dummy in range(1):
        del lpos31[randrange(len(lpos31))]
    for i in range(4):
        cor.append(r"\psline[linestyle=dashed, linecolor=DarkRed](0, %s)(%s, %s)(%s, 0)" % (val_exo3[lpos31[i]][1], val_exo3[lpos31[i]][0], val_exo3[lpos31[i]][1], val_exo3[lpos31[i]][0]))
    cor.extend([r"\end{pspicture*}", r"\end{center}", r"\begin{enumerate}"])
    shuffle(lpos31)
    lquest = [u"Quelle est l'image de $%s$ par la fonction $k$ ?" % val_exo3[lpos31[0]][0],
              u"Donner un antécédent de %s par la fonction $k$." % val_exo3[lpos31[1]][1],
              u"Compléter : $k\\,(%s)=\\ldots\\ldots$" % val_exo3[lpos31[2]][0],
              u"Compléter : $k\\,(\\ldots\\ldots)=%s$" % val_exo3[lpos31[3]][1]]
    lpos32 = [0, 1, 2, 3]
    shuffle(lpos32)
    for i in range(4):
        exo.append(u"\\item %s" % lquest[lpos32[i]])
        if lpos32[i] == 0:
            cor.append(u"\\item L'image de $%s$ par la fonction $k$ est $\\mathbf{%s}$." % (val_exo3[lpos31[0]][0], val_exo3[lpos31[0]][1]))
        elif lpos32[i] == 1:
            cor.append(u"\\item Un antécédent de $%s$ par la fonction $k$ est $\\mathbf{%s}$." % (val_exo3[lpos31[1]][1], val_exo3[lpos31[1]][0]))
        if lpos32[i] == 2:
            cor.append(u"\\item $k\\,(%s)=\\mathbf{%s}$." % (val_exo3[lpos31[2]][0], val_exo3[lpos31[2]][1]))
        elif lpos32[i] == 3:
            cor.append(u"\\item $k\\,(\\mathbf{%s})=%s$." % (val_exo3[lpos31[3]][0], val_exo3[lpos31[3]][1]))
    exo.append(r"\end{enumerate}")
    cor.append(r"\end{enumerate}")



    exo.append(r"\end{enumerate}")
    exo.append(r"\end{multicols}")
    cor.append(r"\end{enumerate}")
    cor.append(r"\end{multicols}")

    return exo, cor

class notion_fonction(LegacyExercise):
    """Bilan sur la notion de fonction"""

    tags = ["Troisième"]
    function = _notion_fonction




# Pyromaths
#
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
#
# Copyright (C) 2018 -- Louis Paternault (spalax@gresille.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

"""Équations du premier degré"""

import random

from pyromaths.ex import Jinja2Exercise
from pyromaths.outils.jinja2utils import facteur

class EquationPremierDegre61(Jinja2Exercise):
    """Résolution d'équations du premier degré à coefficients entiers."""
    tags = ['Troisième', 'Équation', 'Premier degré']

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        a = random.choice([1, -1]) * random.randint(2, 9)
        b = random.choice([1, -1]) * random.randint(2, 9)
        c = random.choice([1, -1]) * random.randint(2, 9)
        d = random.choice([1, -1]) * random.randint(2, 9)

        calculs = [
                r"{a}x{b:+d} &= {c}x{d:+d}\\".format(a=a, b=b, c=c, d=d),
                r"{a}x{c:+d}x &= {d}{b:+d}\\".format(a=a, b=-b, c=-c, d=d),
            ]
        if a-c == 0:
            calculs.append(r"0 &= {}\\".format(d-b))
            if d == b:
                conclusion = u"Puisque $0=0$ est toujours vrai, alors l'équation a une infinité de solutions : tous les nombres réels sont des solutions."
            else:
                conclusion = u"Puisque $0={}$ est toujours faux, alors l'équation n'a aucune solution.".format(d-b)
        elif a-c == -1:
            calculs.append(r"-x &= {}\\".format(d-b))
            calculs.append(r"(-1)\times -x&= (-1)\times {}\\".format(d-b))
            calculs.append(r"x&={}\\".format(b-d))
            conclusion = r"L'unique solution est $x = {}$.".format(b-d)
        elif a-c == 1:
            calculs.append(r"x &= {}\\".format(d-b))
            conclusion = r"L'unique solution est $x = {}$.".format(d-b)
        else:
            calculs.append(r"{}x &= {}\\".format(a-c, d-b))
            calculs.append(r"x&=\frac{{ {} }}{{ {} }}\\".format(d-b, a-c))
            solution = facteur(float(d-b)/float(a-c), "2")
            if 100*(float(d-b)/float(a-c)) == int(100 * float(d-b)/float(a-c)):
                calculs.append(r"x&={}\\".format(solution))
                conclusion = r"L'unique solution est $x = {}$.".format(solution)
            else:
                calculs.append(r"x&\simeq{}\\".format(solution))
                conclusion = r"L'unique solution est $x \simeq {}$.".format(solution)
        self.context = {
            "a": a,
            "b": b,
            "c": c,
            "d": d,
            "calculs": calculs,
            "conclusion": conclusion,
            }



#! /usr/bin/env python3

"""Méta-information about pyromaths."""

# Ce fichier ne doit avoir *aucun* import à part ceux de la librairie standard.

import textwrap
import time

VERSION = '21.8.1'
AUTHOR = "Jérôme Ortais"
AUTHOR_EMAIL = "jerome.ortais@pyromaths.org"
COPYRIGHT_YEAR = time.strftime('%Y')
COPYRIGHT_HTML = textwrap.dedent("""\
        © 2006 – {} Jérôme Ortais
        <br/>
        <span style=" font-size:small;">Pyromaths est distribué sous licence GPL.</span>
        """).format(COPYRIGHT_YEAR)
WEBSITE = 'http://www.pyromaths.org/'



#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#
#----------------------------------------------------------------------
# Pyromaths : Initiation au calcul littéral
#----------------------------------------------------------------------
from pyromaths.outils.Priorites3 import texify, priorites, splitting
from random import randrange, shuffle
from pyromaths.ex import LegacyExercise

def valeurs_reduire():
    """Travail sur les bases du calcul littéral en quatrième"""
    var = "atxy"
    var = var[randrange(len(var))]
    op = "+*-*"[randrange(4)]
    if op == "*":
        deg1 = randrange(3)
        if deg1 == 2: deg2 = 0
        elif deg1 == 1: deg2 = randrange(2)
        else: deg2 = randrange(1, 3)
    else:
        deg1 = randrange(1, 3)
        deg2 = [i for i in range(3)]
        deg2.extend([deg1] * 7)
        shuffle(deg2)
        deg2 = deg2.pop(randrange(len(deg2)))
    a1, a2 = 0, 0
    while not a1 or not a2:
        a1 = randrange(-10, 11)
        a2 = randrange(-10, 11)
    p1 = "Polynome(\"%s%s^%s\", details=3)" % (a1, var, deg1)
    p2 = "Polynome(\"%s%s^%s\", details=3)" % (a2, var, deg2)
    return p1 + op + p2

def _reduire():
    """Travail sur les bases du calcul littéral en quatrième"""
    exo = ["\\exercice", u"Réduire, si possible, les expressions suivantes :",
           "\\begin{multicols}{3}\\noindent", "  \\begin{enumerate}"]
    cor = ["\\exercice*", u"Réduire, si possible, les expressions suivantes :",
           "\\begin{multicols}{3}\\noindent", "  \\begin{enumerate}"]
    for dummy in range(9):
        a = valeurs_reduire()
        solve = [a]
        exo.append("\\item $\\thenocalcul = " + texify(solve)[0] + "$")
        cor.append("\\item $\\thenocalcul = " + texify(solve)[0] + "$")
        solve = priorites(a)
        solve.insert(0, a)
        solve = texify(solve)
        if len(solve) > 1:
            for e in solve[1:]:
                cor.append("\\[\\thenocalcul = " + e + "\\]")
        exo.append("\\stepcounter{nocalcul}")
        cor.append("\\stepcounter{nocalcul}")
    exo.extend(["  \\end{enumerate}", "\\end{multicols}"])
    cor.extend(["  \\end{enumerate}", "\\end{multicols}"])
    return (exo, cor)

class reduire(LegacyExercise):
    """Bases du calcul littéral"""

    tags = ["Quatrième"]
    function = _reduire

def _distributivite():
    """Crée un exercice permettant de s'entrainer sur la distributivité
    """
    lcalc, expr = [], []
    for i in range(2):
        tmp = ['Polynome([[%s, 1]], details=3)' % randrange(2, 10), 'Polynome([[%s, 0]], details=3)' % randrange(2, 10)]
        shuffle(tmp)
        lcalc.append(["*".join(tmp)])
        lcalc[i].extend(priorites(lcalc[i][0]))
        lcalc[i][0] = splitting(lcalc[i][0])
        expr.append(texify(lcalc[i]))
    tmp = [0, 1, 2]
    for i in range(2, 5):
        k = 'Polynome([[%s, 0]], details=3)' % randrange(2, 11)
        if i > 1:
            a = valeurs_reduire_somme(2)
        else:
            sgn = (-1) ** randrange(2)
            a = [[sgn, 1] for dummy2 in range(randrange(2, 5))]
            a.append([randrange(1, 11) * (-1) ** randrange(2), 0])
            a = 'Polynome(%s, var=\'x\', details=3)' % a
        exp = tmp.pop(randrange(len(tmp)))
        if exp != 2:
            b = 'Polynome([[%s, %s]], details=3)' % (randrange(1, 11) * (-1) ** randrange(3), exp)
        else:
            b = valeurs_reduire_somme(2)

        lpoly = [a, k]
        shuffle(lpoly)
        if randrange(2):lpoly.insert(0, b)
        else:lpoly.append(b)
        if b == lpoly[0]:
            lcalc.append(['%s+%s*%s' % (lpoly[0], lpoly[1], lpoly[2])])
        else:
            lcalc.append(['%s*%s+%s' % (lpoly[0], lpoly[1], lpoly[2])])
        lcalc[i].extend(priorites(lcalc[i][0]))
        lcalc[i][0] = splitting(lcalc[i][0])
        expr.append(texify(lcalc[i]))
    exo = ["\\exercice", u"Développer et réduire chacune des expressions littérales suivantes :"]
    exo.append("\\begin{multicols}{2}")
    cor = ["\\exercice*", u"Développer et réduire chacune des expressions littérales suivantes :"]
    cor.append("\\begin{multicols}{2}")

    for i in range(len(lcalc)):
        cor.append('\\\\\n'.join(['$%s=%s$' % (chr(i + 65), expr[i][j]) for j in range(len(expr[i]) - 1)]))
        cor.append('\\\\')
        cor.append('\\fbox{$%s=%s$}\\\\\n' % (chr(i + 65), expr[i][-1]))
    exo.append('\\\\\n'.join(['$%s=%s$' % (chr(i + 65), expr[i][0]) for i in range(len(expr))]))
    exo.append("\\end{multicols}")
    cor.append("\\end{multicols}")
    return exo, cor

class distributivite(LegacyExercise):
    """Distributivité"""

    tags = ["Quatrième"]
    function = _distributivite

def _double_distributivite():
    """Crée un exercice permettant de s'entrainer sur la double distributivité
    """
    lcalc, expr = [], []
    for i in range(2):
        tmp = ['Polynome([[%s, 1]], details=3)' % ([1, randrange(2, 10)][i]), 'Polynome([[%s, 1]], details=3)' % randrange(2, 10)]
        shuffle(tmp)
        lcalc.append(["*".join(tmp)])
        lcalc[i].extend(priorites(lcalc[i][0]))
        lcalc[i][0] = splitting(lcalc[i][0])
        expr.append(texify(lcalc[i]))
    tmp = [0, 1, 2]
    for i in range(2, 5):
        a = valeurs_reduire_somme(2)
        b = valeurs_reduire_somme(2)
        exp = tmp.pop(randrange(len(tmp)))
        if exp != 1:
            c = 'Polynome([[%s, %s]], details=3)' % (randrange(1, 11) * (-1) ** randrange(3), exp)
        else:
            c = valeurs_reduire_somme(2)
        lpoly = [a, b]
        if randrange(2): lpoly.insert(0, c)
        else: lpoly.append(c)
        if c == lpoly[0]:
            lcalc.append(['%s+%s*%s' % (lpoly[0], lpoly[1], lpoly[2])])
        else:
            lcalc.append(['%s*%s+%s' % (lpoly[0], lpoly[1], lpoly[2])])
        lcalc[i].extend(priorites(lcalc[i][0]))
        lcalc[i][0] = splitting(lcalc[i][0])
        expr.append(texify(lcalc[i]))
    exo = ["\\exercice", u"Développer et réduire chacune des expressions littérales suivantes :"]
    exo.append("\\begin{multicols}{2}")
    cor = ["\\exercice*", u"Développer et réduire chacune des expressions littérales suivantes :"]
    cor.append("\\begin{multicols}{2}")
    for i in range(len(lcalc)):
        if i == 2: cor.append("\\end{multicols}")
        cor.append('\\\\\n'.join(['$%s=%s$' % (chr(i + 65), expr[i][j]) for j in range(len(expr[i]) - 1)]))
        cor.append('\\\\')
        cor.append('\\fbox{$%s=%s$}\\\\\n' % (chr(i + 65), expr[i][-1]))
    exo.append('\\\\\n'.join(['$%s=%s$' % (chr(i + 65), expr[i][0]) for i in range(len(expr))]))
    exo.append("\\end{multicols}")
    return exo, cor

class double_distributivite(LegacyExercise):
    """Double distributivité"""

    tags = ["Quatrième"]
    function = _double_distributivite

def _soustraction():
    """Réduction d'expressions du type 5a+3-(5+a)
    """
    lcalc, expr = [], []
    sgn = ['-', '-', '-', '-', '+', '+']
    for i in range(len(sgn)):
        a = valeurs_reduire_somme(2)
        b = 'Polynome([[%s,0]], var=\'x\', details=3)' % (randrange(2, 11) * (-1) ** randrange(2))
        c = 'Polynome([[%s,1]], var=\'x\', details=3)' % (randrange(2, 11) * (-1) ** randrange(2))
        poly = [a, b, c]
        shuffle(poly)
        signe = sgn.pop(randrange(len(sgn)))
        poly.insert(poly.index(a), signe)
        if signe == '+':
            poly.insert(poly.index(a), '(')
            poly.insert(poly.index(a) + 1, ')')
        poly.insert(poly.index(b), '+')
        poly.insert(poly.index(c), '+')
        if poly[0] == '+': del poly[0]
        lcalc.append(["".join(poly)])
        lcalc[i].extend(priorites(lcalc[i][0]))
        lcalc[i][0] = splitting(lcalc[i][0])
        expr.append(texify(lcalc[i]))
    exo = ["\\exercice", u"Réduire chacune des expressions littérales suivantes :"]
    exo.append("\\begin{multicols}{2}")
    cor = ["\\exercice*", u"Réduire chacune des expressions littérales suivantes :"]
    cor.append("\\begin{multicols}{2}")
    for i in range(len(lcalc)):
        cor.append('\\\\\n'.join(['$%s=%s$' % (chr(i + 65), expr[i][j]) for j in range(len(expr[i]) - 1)]))
        cor.append('\\\\')
        cor.append('\\fbox{$%s=%s$}\\\\\n' % (chr(i + 65), expr[i][-1]))
    exo.append('\\\\\n'.join(['$%s=%s$' % (chr(i + 65), expr[i][0]) for i in range(len(expr))]))
    exo.append("\\end{multicols}")
    cor.append("\\end{multicols}")
    return exo, cor

class soustraction(LegacyExercise):
    """Soustraire une expression entre parenthèses"""

    tags = ["Quatrième"]
    function = _soustraction

def valeurs_reduire_somme(nbval=4):
    """Réduire une somme de quatre monômes de degrés 0 et 1"""
    var = "x"
    #===========================================================================
    # var = var[randrange(len(var))]
    #===========================================================================
    l = [[randrange(1, 11) * (-1) ** randrange(2), (i + 1) % 2] for i in range(nbval)]
    return "Polynome(%s, var=\"%s\", details=3)" % (l, var)

def _exo_comptable():
    """Exercice tiré de l'excellent ouvrage Des maths ensemble et pour chacun quatrième
    """
    exo = ["\\exercice", u"Le principe est le suivant : l'extrémité de chaque flèche indique la somme de la ligne ou de la colonne correspondante. Compléter, sachant que $x$ représente un nombre quelconque et que le contenu des deux cases grises doit être le même.\\par"]
    cor = ["\\exercice*", u"Le principe est le suivant : l'extrémité de chaque flèche indique la somme de la ligne ou de la colonne correspondante. Compléter, sachant que $x$ représente un nombre quelconque et que le contenu des deux cases grises doit être le même.\\par"]
    lexo = [valeurs_reduire_somme(2) for dummy in range(8)]
    lcalc = [["%s+%s" % (lexo[4], lexo[0])], ["%s+%s" % (lexo[5], lexo[1])], ["%s+%s" % (lexo[6], lexo[2])], ["%s+%s" % (lexo[7], lexo[3])], \
             ["%s+%s+%s+%s" % tuple(lexo[0:4])], [ "%s+%s+%s+%s" % tuple(lexo[4:])]]
    for i in range(6):
        sol = priorites(lcalc[i][0])
        lcalc[i].extend(["".join(sol[j]) for j in range(len(sol))])
    lcalc.append(["%s+%s+%s+%s" % (lcalc[0][-1], lcalc[1][-1], lcalc[2][-1], lcalc[3][-1])])
    lcalc.append(["%s+%s" % (lcalc[4][-1], lcalc[5][-1])])
    for i in range(6, 8):
        sol = priorites(lcalc[i][0])
        lcalc[i].extend(["".join(sol[j]) for j in range(len(sol))])
    for i in range(8):
        for j in range(len(lcalc[i])):
            lcalc[i][j] = splitting(lcalc[i][j])
    expr = [texify(lcalc[i]) for i in range(8)]
    # expr = [texify([splitting(lexo[i])]) for i in range(8)]
    txt = []
    # txt.append(r"\psset{xunit=1.1cm,yunit=1.5cm}")
    txt.append(r"\begin{center}")
    txt.append(r"\begin{pspicture}(16,6)")
    txt.append(r"\psframe[fillstyle=solid,fillcolor=Gray](0,0)(2,1)")
    txt.append(r"\psframe[fillstyle=solid,fillcolor=Gray](14,5)(16,6)")
    txt.append(r"\psframe(14,2)(16,3)")
    txt.append(r"\psframe(14,3)(16,4)")
    txt.append(r"\multido{\i=4+2}{4}{")
    txt.append(r"\rput(\i,0){\psframe(0,0)(2,1)\psline[linewidth=2pt]{<-}(1,1.2)(1,1.8)}")
    txt.append(r"\rput(\i,2){\psframe(0,0)(2,1)}")
    txt.append(r"\rput(\i,3){\psframe(0,0)(2,1)}}")
    txt.append(r"\psline[linewidth=2pt]{<-}(2.2,.5)(3.8,.5)")
    txt.append(r"\psline[linewidth=2pt]{->}(12.2,2.5)(13.8,2.5)")
    txt.append(r"\psline[linewidth=2pt]{->}(12.2,3.5)(13.8,3.5)")
    txt.append(r"\psline[linewidth=2pt]{<-}(15,4.8)(15,4.2)")
    txt.append(r"\rput(5,2.5){$%s$}" % texify([splitting(lexo[0])])[0])
    txt.append(r"\rput(7,2.5){$%s$}" % texify([splitting(lexo[1])])[0])
    txt.append(r"\rput(9,2.5){$%s$}" % texify([splitting(lexo[2])])[0])
    txt.append(r"\rput(11,2.5){$%s$}" % texify([splitting(lexo[3])])[0])
    txt.append(r"\rput(5,3.5){$%s$}" % texify([splitting(lexo[4])])[0])
    txt.append(r"\rput(7,3.5){$%s$}" % texify([splitting(lexo[5])])[0])
    txt.append(r"\rput(9,3.5){$%s$}" % texify([splitting(lexo[6])])[0])
    txt.append(r"\rput(11,3.5){$%s$}" % texify([splitting(lexo[7])])[0])
    exo.extend(txt)
    txt.append(r"\rput(5,.5){$%s$}" % expr[0][-1])
    txt.append(r"\rput(7,.5){$%s$}" % expr[1][-1])
    txt.append(r"\rput(9,.5){$%s$}" % expr[2][-1])
    txt.append(r"\rput(11,.5){$%s$}" % expr[3][-1])
    txt.append(r"\rput(15,2.5){$%s$}" % expr[4][-1])
    txt.append(r"\rput(15,3.5){$%s$}" % expr[5][-1])
    txt.append(r"\rput(1,.5){$%s$}" % expr[6][-1])
    txt.append(r"\rput(15,5.5){$%s$}" % expr[7][-1])
    cor.extend(txt)
    exo.append("\\end{pspicture}\n\\end{center}")
    cor.append("\\end{pspicture}\n\\end{center}")
    cor.append(u"\\subsubsection*{Ligne du bas :}")
    cor.append(u"\\begin{multicols}{4}")
    for i in range(4):
        cor.append('\\\\\n'.join(['$%s=%s$' % (chr(i + 65), expr[i][j]) for j in range(len(expr[i]) - 1)]))
        cor.append('\\\\')
        cor.append('\\fbox{$%s=%s$}\\\\\n' % (chr(i + 65), expr[i][-1]))
    cor.append(u"\\end{multicols}")
    cor.append(u"\\subsubsection*{Colonne de droite :}")
    cor.append(u"\\begin{multicols}{2}")
    for i in range(4, 6):
        cor.append('\\\\\n'.join(['$%s=%s$' % (chr(i + 65), expr[i][j]) for j in range(len(expr[i]) - 1)]))
        cor.append('\\\\')
        cor.append('\\fbox{$%s=%s$}\\\\\n' % (chr(i + 65), expr[i][-1]))
    cor.append(u"\\end{multicols}")
    cor.append(u"\\subsubsection*{Cases grises :}")
    cor.append(u"\\begin{multicols}{2}")
    for i in range(6, 8):
        cor.append('\\\\\n'.join(['$%s=%s$' % (chr(i + 65), expr[i][j]) for j in range(len(expr[i]) - 1)]))
        cor.append('\\\\')
        cor.append('\\fbox{$%s=%s$}\\\\\n' % (chr(i + 65), expr[i][-1]))
    cor.append(u"\\end{multicols}")

    return exo, cor

class exo_comptable(LegacyExercise):
    """Réduire des expressions littérales"""

    tags = ["Quatrième"]
    function = _exo_comptable

#===============================================================================
#
# def reduire_expressions():
#     """Fait double emploi avec l'exercice du comptable"""
#     """Travail sur les bases du calcul littéral en quatrième"""
#     exo = ["\\exercice", u"Réduire les expressions littérales suivantes :",
#            "\\begin{multicols}{2}"]
#     cor = ["\\exercice*", u"Réduire les expressions littérales suivantes :",
#            "\\begin{multicols}{2}"]
#     lexo = [valeurs_reduire_somme() for dummy in range(4)]
#     expr = [texify([splitting(lexo[i])]) for i in range(4)]
#     for i in range(4):
#         expr[i].extend(texify(priorites(lexo[i])))
#         exo.append('\\\\\n$%s=%s$' % (chr(i + 65), expr[i][0]))
#         cor.append('\\\\\n'.join(['$%s=%s$' % (chr(i + 65), expr[i][j]) for j in range(len(expr[i]) - 1)]))
#         cor.append('\\\\')
#         cor.append('\\fbox{$%s=%s$}\\\\\n' % (chr(i + 65), expr[i][-1]))
#     exo.append("\\end{multicols}")
#     cor.append("\\end{multicols}")
#     return exo, cor
#
# # eduire_expressions.description = u'Réduire des expressions littérales'
#===============================================================================



# Pyromaths
#
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
#
# Copyright (C) 2018 -- Louis Paternault (spalax@gresille.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

"""Classes et fonctions de jinja2 adaptées pour LaTeX."""

from jinja2 import *

class LatexEnvironment(Environment):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.block_start_string = '(*'
        self.block_end_string = '*)'
        self.variable_start_string = '(('
        self.variable_end_string = '))'
        self.comment_start_string = '(% '
        self.comment_end_string = ' %)'
        self.trim_blocks = True
        self.lstrip_blocks = True



#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#
from pyromaths.outils.decimaux import decimaux
from pyromaths.outils import Priorites3
from pyromaths.classes.SquareRoot import SquareRoot
from pyromaths.classes.Fractions import Fraction

from random import *
from functools import reduce

class Polynome:
    """ Cette classe crée la notion de polynômes.
    
        Si ``var == None`` alors la variable est ``x``.
        
        *details* donne le niveau de détails attendus dans les développements et réductions :

        - 0 : pas de détails : 3x+4x => 7x
        - 1 : ordonne avant de réduire
        - 2 : ordonne puis factorise les réductions : 3x+4x=(3+4)x
        - 3 : comme 2 et détaille les produits : 2x*3x=2*3*x*x

        >>> from pyromaths.classes.PolynomesCollege import Polynome
        >>> repr(Polynome([[2,2],[3,1],[4,0]], 'z'))
        Polynome([[2, 2], [3, 1], [4, 0]], "z", 0)
        >>> repr(Polynome("2y^2+3y+4"))
        Polynome([[2, 2], [3, 1], [4, 0]], "y", 0)
        >>> repr(Polynome([[1, 1], [2, 2]]))
        Polynome([[1, 1], [2, 2]], "x", 0)
        >>> repr(Polynome("Fraction(1,7)x^2-Fraction(3,8)x-1"))
        Polynome([[Fraction(1, 7), 2], [Fraction(-3, 8), 1], [-1, 0]], "x", 0)
    """

    def __init__(self, monomes, var=None, details=0):
        """Crée un polynôme."""
        monomes = monomes or '0'  # monômes du polynôme, par défaut un polynôme nul
        if isinstance(monomes, Polynome):
            self.monomes = monomes.monomes
            self.var = monomes.var
            self.details = monomes.details
        elif isinstance(monomes, str):
            # Gère la construction des polynôme à partir d'une chaîne de caractères
            self.details = details
            listmonomes = []
            splitted = Priorites3.splitting(monomes)
            m_coef, var, coef = None, None, ''
            while splitted:
                extract = splitted.pop(0)
                if extract in '+-':
                    if m_coef:
                        listmonomes.append([m_coef, 0])
                        m_coef = None
                    coef = extract
                elif Priorites3.EstNombre(extract):
                    m_coef = eval(coef + extract)
                else:
                    if var and extract != var:
                        raise ValueError(u'Le nom de la variable (%s) est incorrect pour le Polynôme %s' % (var, monomes))
                    if var is None: var = extract
                    if splitted and splitted[0] == '^':
                        splitted.remove('^')
                        m_deg = eval(splitted.pop(0))
                    else:
                        m_deg = 1
                    if m_coef is None:
                        if coef == '-':
                            m_coef = -1
                        else:
                            m_coef = 1
                    listmonomes.append([m_coef, m_deg])
                    m_coef = None
            if m_coef: listmonomes.append([m_coef, 0])
            if listmonomes: self.monomes = listmonomes
            else: self.monomes = [[0, 0]]
            self.var = var or 'x'
        else:
            # Supprime les monomes nuls :
            for k in range(len(monomes) - 1, -1, -1):
                if monomes[k][0] == 0: monomes.pop(k)
            if not monomes: monomes = [[0, 0]]
            self.monomes = monomes
            self.var = var or 'x'  # Variable par défaut
            self.details = details


    def __repr__(self):
        """**repr**\ (*object*)

        Renvoie une chaîne de caractère représentant un :mod:`Polynome`
        évaluable pour créer un :mod:`Polynome`.

            >>> from pyromaths.classes.PolynomesCollege import Polynome
            >>> repr(Polynome([[2,2],[3,1],[4,0]], 'z'))
            Polynome([[2, 2], [3, 1], [4, 0]], "z", 0)

        :rtype: string
        """
        return "Polynome(%s, \"%s\", %s)" % (self.monomes, self.var, self.details)

    def __str__(self):
        r"""**str**\ (*object*)

        Renvoie une version LaTeX du polynôme.
            >>> from pyromaths.classes.PolynomesCollege import Polynome
            >>> str(Polynome([[2,2],[3,1],[4,0]], 'z'))
            2\,z^{2}+3\,z+4
            >>> str(Polynome("y^2-Fraction(3,2)y-1"))
            y^{2}-\dfrac{3}{2}\,y-1
            >>> from pyromaths.classes.Fractions import Fraction
            >>> Polynome([[1, 1], [Fraction(-5, 1), 0]])
            x-5

        :rtype: string
        """
        def print_coef(coef):
            """Gère le format du coef
            """
            if isinstance(coef, (float, int)):
                if coef > 0: return "+" + decimaux(coef)
                else: return decimaux(coef)
            if isinstance(coef, Fraction):
                if isinstance(coef.n, int) and isinstance(coef.d, int) and coef.n < 0 and coef.d > 0:
                    return "-" + str(Fraction(-coef.n, coef.d, coef.code))
                return "+" + str(coef)
            if isinstance(coef, SquareRoot):
                if len(coef) > 1: coef = str(coef)
                else:
                    if coef[0][0] > 0:
                        return '+' + str(coef)
                    else:
                        return str(coef)
            if isinstance(coef, str):
                texte = "(" + "".join(Priorites3.texify([Priorites3.splitting(coef)])) + ")"
                if texte[0] != "-": return "+" + texte
                else: return texte
            raise ValueError(u'Not Implemented : Coefficient %s' % coef)

        var = self.var
        s = ""
        for m in self.monomes:
            if isinstance(m, list):
                if m[1] > 1:
                    # Monôme de degré au moins 2
                    if m[0] == 1:
                        s = s + "+" + var + "^{" + str(m[1]) + "}"
                    elif m[0] == -1:
                        s = s + "-" + var + "^{" + str(m[1]) + "}"
                    else:
                        s = s + print_coef(m[0]) + r"\," + var + "^{" + str(m[1]) + "}"
                elif m[1] == 1:
                    # Monôme de degré 1
                    if m[0] == 1:
                        s = s + "+" + var
                    elif m[0] == -1:
                        s = s + "-" + var
                    else:
                        s = s + print_coef(m[0]) + r"\," + var
                else:
                    # Monôme de degré 0
                    s = s + print_coef(m[0])
        # supprime le + en début de séquence
        s = s.lstrip("+")
        if not s: s = "0"
        return s

    def __call__(self, valeur):
        """**__call__**\ (*object*\ ,*valeur*)

        ``p.__call__(valeur)`` est équivalent à ``p(valeur)``
        Retourne l'expression numérique du polynôme pour sa variable égale à
        valeur.

            >>> from pyromaths.classes.PolynomesCollege import Polynome
            >>> Polynome("4+x-x^2+x^3-2x^5")(-5)
            4-5-(-5)**2+(-5)**3-2*(-5)**5

        :param: valeur
        :type: integer, float or Fraction
        :rtype: string
        """
        if Polynome.degre(self) == 0:
            return self.monomes[0][0] or '0'
        if valeur == 0:
            retour = ''
            for m in self.monomes:
                if m[1] == 0: retour += '+%r' % m[0]
            return retour.lstrip('+').replace('+-', '-') or '0'
        if valeur < 0 and isinstance(valeur, (float, int)): nb = "(%s)" % valeur
        else: nb = repr(valeur)
        retour = ''
        for m in self.monomes:
            if m[1] == 0: retour += '+%r' % m[0]
            elif m[1] == 1:
                if m[0] == 1:
                    retour += '+%r' % valeur
                elif m[0] == -1:
                    retour += '-' + nb
                else:
                    retour += '+%r*' % m[0] + nb
            else:
                if m[0] == 1:
                    retour += '+' + nb + '**%s' % m[1]
                elif m[0] == -1:
                    retour += '-' + nb + '**%s' % m[1]
                else:
                    retour += '+%r*' % m[0] + nb + '**%s' % m[1]
        return retour.lstrip('+').replace('+-', '-')

    def derive(self):
        if Polynome.degre(self) < 1:
            return Polynome([0, 0], self.var, self.details)
        else:
            p = []
            for m in self:
                if m[1] > 0:
                    p.append([m[0] * m[1], m[1] - 1])
            return Polynome(p, self.var, self.details)


    def __getitem__(self, i):
        """*object*\ .\ **__getitem__**\ (*integer*)

        Renvoie le i ème monome du polynôme.

            >>> from pyromaths.classes.PolynomesCollege import Polynome
            >>> Polynome("2y+3y^2+4")[1]
            [3, 2]

        :rtype: list
        """
        return self.monomes[i]

    def __delitem__(self, i):
        """*object*\ .\ **__delitem__**\ (*integer*)

        Renvoie le polynôme privé du i ème monome.

            >>> from pyromaths.classes.PolynomesCollege import Polynome
            >>> p = Polynome("2y+3y^2+4")
            >>> del p[1]
            >>> repr(p)
            Polynome([[2, 1], [4, 0]], "y", 0)

        :rtype: Polynome
        """
        m = self.monomes
        del m[i]
        return Polynome(m, self.var, self.details)

    def __iadd__(self, other):
        """*object*\ .\ **__iadd__**\ (*other*)

        ``p.__iadd__(q)`` est équivalent à ``p += q``

            >>> from pyromaths.classes.PolynomesCollege import Polynome
            >>> p=Polynome("2y+3y^2+4")
            >>> p += Polynome('-y+6')
            >>> repr(p)
            Polynome([[2, 1], [3, 2], [4, 0], [-1, 1], [6, 0]], "y", 0)

        :rtype: Polynome
        """
        other = self._convert_other(other)
        m1, m2 = [m for m in self.monomes], [m for m in other.monomes]
        m = []
        m1.extend(m2)
        for monomes in m1:
            if monomes[0]: m.append(monomes)
        return Polynome(m, self.var, self.details)

    def __eq__(self, other):
        """*object*\ .\ **__eq__**\ (*other*)

        ``p.__eq__(q)`` est équivalent à ``p == q``
        Renvoie True si deux polynômes sont égaux. Ne tient pas compte de
        l'ordre des monômes.

            >>> from pyromaths.classes.PolynomesCollege import Polynome
            >>> Polynome("2y+3y^2+4") == Polynome('-y+6')
            False
            >>> Polynome('3x+5x+4')== Polynome('8x+4')
            False

        :rtype: boolean
        """
        other = self._convert_other(other)
        return sorted(self.monomes, key=lambda x: (-x[1], x[0])) == \
               sorted(other.monomes, key=lambda x: (-x[1], x[0]))

    def __ne__(self, other):
        """*object*\ .\ **__ne__**\ (*other*)

        ``p.__ne__(q)`` est équivalent à ``p != q``
        Renvoie True si deux polynômes sont différents. Ne tient pas compte de
        l'ordre des monômes.

            >>> from pyromaths.classes.PolynomesCollege import Polynome
            >>> Polynome("2y+3y^2+4") != Polynome('-y+6')
            True

        :rtype: boolean
        """
        return not (self == other)

    def __add__(self, *others):
        """*object*\ .\ **__add__**\ (*\ \*others*)

        ``p.__add__(q)`` est équivalent à ``p + q``  calcule la somme de
        polynômes.

        *other* peut être une chaîne représentant un polynôme.

            >>> from pyromaths.classes.PolynomesCollege import Polynome
            >>> repr(Polynome("2y+3y^2+4", details=2)+Polynome('-y+6', details=2))
            Polynome([[3, 2], [2, 1], [-1, 1], [4, 0], [6, 0]], "y", 2)

        :param: other
        :type: Polynome ou string *évaluable comme Polynome*
        :rtype: Polynome
        """
#         if isinstance(other, str):
#             other = eval(other)
        for other in others:
            other = self._convert_other(other)
            self +=other
        return self.nreduction()

    def __radd__(self, other):
        """*object*\ .\ **__radd__**\ (*other*)

        ``p.__radd__(q)`` est équivalent à ``q + p``

        >>> from pyromaths.classes.PolynomesCollege import Polynome
        >>> from pyromaths.classes.Fractions import Fraction
        >>> repr(Fraction(5,4)+Polynome("3x"))
        Polynome([[3, 1], [Fraction(5, 4), 0]], "x", 0)

        :param: other
        :type: Polynome ou string *évaluable comme Polynome*
        :rtype: Polynome
        """
        if isinstance(other, str):
            other = eval(other)
        other = self._convert_other(other)
        return other + self

    def __sub__(self, *others):
        """*object*\ .\ **__sub__**\ (*other*)

        ``p.__sub__(q)`` est équivalent à ``p + q``  calcule la somme de
        polynômes.

        *other* peut être une chaîne représentant un polynôme.

            >>> from pyromaths.classes.PolynomesCollege import Polynome
            >>> repr(Polynome("2y+3y^2+4")-Polynome('-y+6'))
            'Polynome([[2, 1], [3, 2], [4, 0]], "y", 0)+Polynome([[1, 1], [-6, 0]], "y", 0)'
            >>> repr(Polynome("3y^2+2y+4")-Polynome('-y+6'))
            'Polynome([[3, 2], [2, 1], [4, 0]], "y", 0)+Polynome([[1, 1], [-6, 0]], "y", 0)'
            >>> repr(Polynome("x+6")-Polynome("3x"))
            Polynome([[-2, 1], [6, 0]], "x", 0)
            >>> repr(Polynome("x+6",details=2)-Polynome("3x",details=2))
            Polynome([[1, 1], [-3, 1], [6, 0]], "x", 2)

        :param: other
        :type: Polynome ou string *évaluable comme Polynome*
        :rtype: Polynome or string
        """
        areduire = False
        if self.ordonne().reductible(): areduire = True
        lother = []
        details, etapes = self.details, False
        for other in others:
            other = self._convert_other(other)
            if other == 0:
                pass
            else:
                if other.ordonne().reductible(): areduire = True
                details = min(details, other.details)
                if len(other) > 1: etapes = True
                lother.append(other)
        if areduire:
            lother = [other.nreduction() for other in lother]
            lother.insert(0, self.nreduction())
            return "-".join([repr(other) for other in lother])
        else:
            calcul = "%r+" % self +"+".join([repr(-other) for other in lother])
            if etapes: return calcul
            else: return eval(calcul)


    def __rsub__(self, other):
        """*object*\ .\ **__rsub__**\ (*other*)

        ``p.__rsub__(q)`` est équivalent à ``q - p``

        >>> from pyromaths.classes.PolynomesCollege import Polynome
        >>> repr(1-Polynome([[-4, 1], [-9, 2], [-5, 0]], "x"))
        'Polynome([[1, 0]], "x", 0)+Polynome([[4, 1], [9, 2], [5, 0]], "x", 0)'
        >>> from pyromaths.classes.Fractions import Fraction
        >>> repr(Fraction(5,4)-Polynome("3x"))
        Polynome([[-3, 1], [Fraction(5, 4), 0]], "x", 0)
        >>> repr(Fraction(5,4)-Polynome("-3x"))
        Polynome([[3, 1], [Fraction(5, 4), 0]], "x", 0)

        :param: other
        :type: Polynome ou string *évaluable comme Polynome*
        :rtype: Polynome
        """
        other = self._convert_other(other)
        if len(self) == 1:
            # Cas où on soustrait un polynôme de longueur 1
            other += -self
            return other.ordonne()
        return "%r+%r" % (other, -self)

    def __neg__(self):
        """*object*\ .\ **__neg__**\ ()

        ``p.__neg__()`` est équivalent à ``-p`` est équivalent à ``p = -p``

        **TODO :** Corriger ce dernier point ; p ne devrait pas être modifié

        Renvoie l'opposé d'un polynôme.

            >>> from pyromaths.classes.PolynomesCollege import Polynome
            >>> repr(-Polynome("2y+3y^2+4"))
            Polynome([[-2, 1], [-3, 2], [-4, 0]], "y", 0)

        :rtype: Polynome
        """
        if self.ordonne().reductible(): return '-%r' % self.nreduction()
        m = [m1 for m1 in self.monomes]
        for i in range(len(m)):
            m[i][0] = -m[i][0]
        return Polynome(m, self.var, self.details)

    def __pos__(self):
        """*object*\ .\ **__pos__**\ ()

        ``p.__pos__()`` est équivalent à ``+p``

        Renvoie le polynôme.

            >>> from pyromaths.classes.PolynomesCollege import Polynome
            >>> repr(+Polynome("2y+3y^2+4"))
            Polynome([[2, 1], [3, 2], [4, 0]], "y", 0)

        :rtype: Polynome
        """
        return self


    def __mul__(self, *others):
        """*object*\ .\ **__mul__**\ (*other*)

        ``p.__mul__(q)`` est équivalent à ``p * q``

        Renvoie une chaîne de caractère ou un objet Polynôme dont les éléments
        sont :

        * le produit de ``p`` et ``q`` si ce sont deux monômes dont un au moins
          a pour coefficient 1.
        * le produit détaillé de ``p`` et ``q`` si ce sont deux monômes ne
          dépendant pas du cas précédent.
        * le développement par distributivité dans les autres cas

        si details est égal à 0, les produits dans les distributivités sont calculés immédiatement,
        sinon les produits sont explicités.

            >>> from pyromaths.classes.PolynomesCollege import Polynome
            >>> repr(Polynome('3x+4', details=0)*Polynome('2x+5', details=0))
            'Polynome([[6, 2]], "x", 0)+Polynome([[15, 1]], "x", 0)+Polynome([[8, 1]], "x", 0)+Polynome([[20, 0]], "x", 0)'
            >>> repr(Polynome('3x', details=1)*Polynome('2x+5', details=1))
            'Polynome([[3, 1]], "x", 1)*Polynome([[2, 1]], "x", 1)+Polynome([[3, 1]], "x", 1)*Polynome([[5, 0]], "x", 1)'
            >>> repr(Polynome('3x', details=3) * Polynome('2x', details=3))
            "Polynome([[3, 0]], var = 'x', details=3)*Polynome([[1, 1]], var = 'x', details=3)*Polynome([[2, 0]], var = 'x', details=3)*Polynome([[1, 1]], var = 'x', details=3)"
            >>> repr(Polynome('3', details=3)*Polynome('x', details=3))
            Polynome([[3, 1]], "x", 3)

        :param: other
        :type: Polynome ou string *évaluable comme Polynome*
        :rtype: string ou Polynome
        """
        def id_rem(p1, p2):
            '''Renvoie 1, 2 ou 3 selon l'identité remarquable trouvée, None sinon'''
            if len(p1) != 2 or len(p2) != 2: return None
            p1, p2 = p1.ordonne(), p2.ordonne()
            m0 = [abs(p1[0][0]), p1[0][1]]
            m1 = [abs(p1[1][0]), p1[1][1]]
            if p1 == p2:
                # 1ère ou 2ème identité remarquable
                return p1 ** 2
            if (p1[0][1] == p2[0][1] and p1[1][1] == p2[1][1]) and (p1[0][0] == p2[0][0] and\
                p1[1][0] == -p2[1][0]) or (p1[0][0] == -p2[0][0] and p1[1][0] == p2[1][0]):
                # 3ème identité remarquable
                if p1[0][0] == p2[0][0]:
                    poly = "%r**2-%r**2" % (Polynome([p1[0]], self.var, self.details), \
                                            Polynome([m1], self.var, self.details))
                else:
                    poly = "%r**2-%r**2" % (Polynome([p1[1]], self.var, self.details), \
                                            Polynome([m0], self.var, self.details))
                return poly
            return None

        if self == 0: return 0
        lother = [self] + list(others)
        for el in lother:
            if el.ordonne().reductible():
                # On commence par réduire la chaîne
                return "*".join([repr(other.nreduction()) for other in lother])

        lcoeff, lexp, reduire, ordonne = [], [], True, True
        for i in range(len(lother)):
            lother[i] = self._convert_other(lother[i])
            if len(lother[i]) == 1:
                # c'est un monôme, il est donc probablement réduit
                coeff, exp = lother[i][0][0], lother[i][0][1]
                if coeff != 1: lcoeff.append(coeff)
                if exp != 0: lexp.append(exp)
                if coeff != 1 and exp != 0: reduire = False
                if lexp and exp < lexp[-1]: ordonne = False
                if lother[i] == 0: return 0
            if len(lother[i]) > 1 or i + 1 == len(lother):
                # Ce n'est pas un monôme, il va donc falloir utiliser la distributivité
                # ou alors la chaine est fini et on calcule
                if i > 1 or len(lother[i]) == 1 or len(lcoeff) > 1 or len(lexp) > 1:
                    # on a multiplié au moins 2 monômes auparavant
                    if lother[i].details == 3 and not reduire:
                        # TODO: cas où un coeff est une str
                        #=======================================================
                        # return "*".join([repr(coeff) for coeff in lcoeff]) + "*" + \
                        #     "*".join(["Polynome([[1, %s]], '%s', %s)" % (exp, self.var, self.details) for exp in lexp]) + \
                        #     "*".join([repr(other.nreduction()) for other in lother[i + 1:]])
                        #=======================================================
                        produit = []
                        for j in lother:
                            if j[0][0] != 1 and j[0][1] > 0:
                                produit.append("*".join(['Polynome([[%s, 0]], var = \'%s\', details=%s)' % (j[0][0], j.var, j.details), \
                                                'Polynome([[1, %s]], var = \'%s\', details=%s)' % (j[0][1], j.var, j.details)]))
                            else:
                                produit.append(repr(j))
                        return "*".join(produit)
                    if lother[i].details == 3 and not ordonne:
                        coeff = "*".join([repr(coeff) for coeff in lcoeff])
                        if coeff: sol = [coeff]
                        else: sol = []
                        exp = "*".join(["Polynome([[1, %s]], '%s', %s)" % (exp, self.var, self.details) for exp in lexp])
                        if exp: sol.append(exp)
                        if lother[i + 1:]: sol.append("*".join([repr(other.nreduction()) for other in lother[i + 1:]]))
                        return "*".join(sol)
                    else:
                        lcoeff.append(1)
                        lexp.append(0)
                        poly = Polynome([[reduce(lambda x, y: x * y, lcoeff), \
                                          reduce(lambda x, y: x + y, lexp)]], self.var, self.details)
                        if i + 1 == len(lother): return poly
                        else: return \
                            "*".join([repr(poly)] + [repr(other.nreduction()) for other in lother[i:]])
                else:
                    # On distribue :
                    if i:
                        # on va utiliser la distributivité simple
                        poly = Polynome([[coeff, exp]], self.var, self.details)
                        if self.details > 0:
                            produit = "+".join([repr(poly) + "*" + repr(Polynome([other], self.var, self.details)) for other in lother[i]])
                            if i + 1 < len(lother): produit = "(" + produit + ")"
                            return "*".join([produit] + [repr(other.nreduction()) for other in lother[i + 1:]])
                        else:
                            produit = "+".join([repr(Polynome([[poly[0][0] * other[0], poly[0][1] + other[1]]], self.var, self.details)) for other in lother[i]])
                            if i + 1 < len(lother): produit = "(" + produit + ")"
                            return "*".join([produit] + [repr(other.nreduction()) for other in lother[i + 1:]])
                    else:
                        # le premier polynome est de longueur au moins 2
                        produit = id_rem(lother[0], lother[1])
                        if produit == None:
                            # Distributivité
                            # TODO: Convertir (a-b) en (a+(-b)) pour details=3
                            #===================================================
                            # if self.details == 3:
                            #     difference, produit = False, []
                            #     for poly in lother:
                            #         if len(poly) > 1:
                            #             for j in range(1, len(poly)):
                            #                 if poly[j][0] < 0:
                            #                     difference = True
                            #                     produit.append("(" + "+".join(['Polynome(%s, var = \'%s\', details=%s)' % (poly[i], poly.var, poly.details) for i in range(len(poly))]) + ")")
                            #                     break
                            #         if not difference: produit.append(repr(poly))
                            #     if difference:
                            #         return "*".join(produit)
                            #===================================================

                            if self.details > 0:
                                produit = "+".join(["+".join([repr(Polynome([premier], self.var, self.details)) + \
                                                              "*" + repr(Polynome([second], self.var, self.details)) \
                                                              for second in lother[1]]) for premier in lother[0]])
                                if i + 2 < len(lother): produit = "(" + produit + ")"
                                return "*".join([produit] + [repr(other.nreduction()) for other in lother[i + 2:]])
                            else:
                                produit = "+".join(["+".join([repr(Polynome([[premier[0] * second[0], premier[1] + second[1]]], self.var, self.details)) \
                                                              for second in lother[1]]) for premier in lother[0]])
                                if i + 2 < len(lother): produit = "(" + produit + ")"
                                return "*".join([produit] + [repr(other.nreduction()) for other in lother[i + 2:]])
                        else:
                            # identité remarquable
                            if i + 2 < len(lother): produit = "(" + produit + ")"
                            return "*".join([produit] + [repr(other.nreduction()) for other in lother[i + 2:]])

    def __rmul__(self, other):
        """Multiplication de *other* (qui n'est pas un polynôme) par *self*
        (qui en est un)

        * Si *other* est une suite de produits de monômes, et *self* un polynôme
          de rang 1, on effectue toutes les multiplications en une fois
        * Sinon, on calcule d'abord @other

        :param: other
        :type: Polynome ou string *évaluable comme Polynome*
        :rtype: string ou Polynome"""
        if isinstance(other, (int, float, Fraction)):
            other = self._convert_other(other)
            return other * self
        if isinstance(other, str):
            from pyromaths.outils.Priorites3 import splitting
            ls = splitting(other)
            if ls.count("+") or ls.count("-"):
                # Réduire @other avant de faire le produit et vérifier s'il faut
                # des parenthèses autour de l'expression @other
                par, besoin = 0, False
                for k in range(len(ls)):
                    if ls[k] == "(": par += 1
                    elif ls[k] == ")": par -= 1
                    elif not par and ls[k] in "+-":
                        besoin = True
                        break
                if besoin: return "(" + other + ")*" + repr(self)
                else: return other + "*" + repr(self)
            elif len(self) > 1:
                return other + "*" + repr(self)
            else:
                index = 0
                for i in range(len(ls)):
                    if "Polynome(" in ls[i]:
                        if len(eval(ls[i])) > 1:
                            return other + "*" + repr(self)
                        elif Polynome.degre(eval(ls[i])) > 0:
                            index = index or i
                index = index or len(ls)
                if self.monomes[0][0] != 1:
                    ls.insert(index, "*")
                    if self.monomes[0][0] < 0: ls.insert(index, "(" + repr(self.monomes[0][0]) + ", %s)" % self.details)
                    else: ls.insert(index, repr(self.monomes[0][0]))
                if self.monomes[0][1] > 0:
                    ls.append("*")
                    ls.append("Polynome([%s], \"%s\", %s)" % ([1, self.monomes[0][1]], self.var, self.details))
                return "".join(ls)
        else: raise ValueError(u"Type non prévu. Bogue en perspective !")

    def _divide(self, other):
        """*object*\ .\ **div**\ (*other*)

        ``p.__truediv__(q)`` est équivalent à ``p / q``

        Renvoie (self // other, self % other) deux polynomes où le premier élément est le quotient et le second
        le reste de la division euclidienne de self par other.

            >>> from pyromaths.classes.PolynomesCollege import Polynome
            >>> Polynome('3x+4', 0)._divide(Polynome('2x+5', 0))
            (Polynome([[Fraction(3, 2), 0]], "x", 0), Polynome([[Fraction(-7, 2), 0]], "x", 0))
            >>> Polynome("x^4+2x^3+3x^2+5x+6")._divide(Polynome("7x+8"))
            (Polynome([[Fraction(1, 7), 3], [Fraction(6, 49), 2], [Fraction(99, 343), 1], [Fraction(923, 2401), 0]], "x", 0), Polynome([[Fraction(7022, 2401), 0]], "x", 0))

        :param: other
        :type: Polynome ou string *évaluable comme Polynome*
        :rtype: Deux Polynome
        """
        other = self._convert_other(other)
        quotient = Polynome([], self.var, self.details)
        reste = Polynome.ordonne(self)
        diviseur = other.ordonne()
        degre_diviseur = diviseur.degre()
        while reste.degre() >= degre_diviseur:
            degre = reste.degre() - degre_diviseur
            if isinstance(reste[0][0], Fraction) or isinstance(diviseur[0][0], Fraction):
                coef = eval(Priorites3.priorites('%r/%r' % (reste[0][0], diviseur[0][0]))[-1][0])
            else:
                if reste[0][0] % diviseur[0][0]:
                    coef = Fraction(reste[0][0], diviseur[0][0])
                else:
                    coef = reste[0][0] // diviseur[0][0]
            quotient.monomes.append([coef, degre])
            reste = eval(Priorites3.priorites('%r-Polynome([[%r, %s]], "%s", %s)*%r' % (reste, coef, degre, self.var, self.details, diviseur))[-1][0])
        return quotient.nreduction(True), reste

    def __mod__(self, other):
        """
        self % other
        """
        other = self._convert_other(other)
        if other is NotImplemented:
            return other
        if not other:
            if self:
                raise ValueError(u'Opération non valide : x % 0')
            else:
                raise ValueError(u'Division non définie : 0 % 0')
        return self._divide(other)[1]

    def __floordiv__(self, other):
        """
        self // other
        """
        other = self._convert_other(other)
        if other is NotImplemented:
            return other
        if not other:
            if self:
                raise ValueError(u'Opération non valide : x // 0')
            else:
                raise ValueError(u'Division non définie : 0 // 0')
        return self._divide(other)[0]

    def __len__(self):
        """*object*\ .\ **__len__**\ ()

        ``p.__len__()`` équivaut à ``len(p)`` et renvoie le nombre de monômes d'un polynôme.

            >>> from pyromaths.classes.PolynomesCollege import Polynome
            >>> len(Polynome("2y+3y^2+4"))
            3

        :rtype: integer
        """
        m = [m1 for m1 in self.monomes]
        m = Polynome.ordonne(Polynome(m, self.var))
        return len(self.monomes)

    def degre(self):
        """**degre**\ (*object*)

        Retourne le degré d'un polynôme, -1 pour le polynôme nul

            >>> from pyromaths.classes.PolynomesCollege import Polynome
            >>> Polynome("2y+3y^2+4").degre()
            2
            >>> Polynome('').degre()
            -1

        :rtype: integer
        """
        if self == 0: return -1
        else: return self.nreduction(True)[0][1]

    def __pow__(self, other):
        """*object*\ .\ **pow**\ (*integer*)

        ``p.__pow__(n)`` équivaut à ``p**n`` et renvoie le développement du
        polynôme p à la puissance n.

        * Si ``n == 1``, alors renvoie l'objet Polynome ;
        * si ``n == 2``, alors renvoie une chaîne avec le développement en
          utilisant une identité remarquable ;
        * si ``n > 1``, renvoie le développement en utilisant la formule du
          binôme de Newton.

        **TODO :** si ``n == 2`` et ``len(p) > 2``, bug

            >>> from pyromaths.classes.PolynomesCollege import Polynome
            >>> repr(Polynome("2y+4")**2)
            'Polynome([[2, 1]], "y", 0)**2+2*Polynome([[2, 1]], "y", 0)*Polynome([[4, 0]], "y", 0)+Polynome([[4, 0]], "y", 0)**2'
            >>> repr(Polynome("2y")**3)
            Polynome([[8, 3]], "y", 0)

        :rtype: string ou Polynome
        """
        if len(self) == 2 and other == 2:
            a0 = self.monomes[0][0]
            b0 = self.monomes[1][0]
            p = a0 * b0
            a = Polynome([[abs(a0), self.monomes[0][1]]], self.var, self.details)
            b = Polynome([[abs(b0), self.monomes[1][1]]], self.var, self.details)
            if p < 0:
                return "%r**2-2*%r*%r+%r**2" % (a, a, b, b)
            else:
                return "%r**2+2*%r*%r+%r**2" % (a, a, b, b)
        elif len(self) == 1:
            return Polynome([[self[0][0] ** other, self[0][1] * other]], self.var)
        else:
            result = self
            for dummy in range(other - 1):
                result = eval(result * self)
            return result

    def reductible(self):
        """**reductible**\ (*object*)

        Retourne True si le polynôme est réductible ou ordonnable, False sinon.

            >>> from pyromaths.classes.PolynomesCollege import Polynome
            >>> Polynome([[2, 1], [3, 2], [4, 0], [-1, 1], [6, 0]], "y").reductible()
            True

        :param type: Polynome
        :rtype: boolean
        """
        #=======================================================================
        # if self != self.reduit():
        #     return True
        # else:
        #     return False
        #=======================================================================
        ldegres = []
        for m in self:
            if m[1] in ldegres: return True
            # Il existe déjà un monome de même degré, le polynome est donc réductible.
            elif ldegres and m[1] > ldegres[-1]: return True
            # Polynome non ordonné
            else: ldegres.append(m[1])
            if isinstance(m[0], str) and Priorites3.effectue_calcul(Priorites3.splitting(m[0])) != [m[0]]: return True
            if isinstance(m[0], Fraction) and repr(m[0].traitement(True)) != repr(m[0]): return True
            if isinstance(m[0], SquareRoot) and repr(m[0].simplifie()) != repr(m[0]): return True
            # une simplification de fraction ou un calcul est à faire
        return False

    def ordonnable(self):
        """**ordonnable**\ (*object*)

        Retourne True si le polynome n'est pas ordonnée, False sinon.

        :param type: Polynome
        :rtype: boolean
        """
        lastdegre = None
        for m in self:
            if lastdegre is not None and m[1] > lastdegre: return True
            else: lastdegre = m[1]
        return False

    def ordonne(self):
        """**ordonne**\ (*object*)

        Retourne une version ordonnée d'un polynome en écrivant en premier les
        monômes de degré supérieur.

            >>> from pyromaths.classes.PolynomesCollege import Polynome
            >>> repr(Polynome([[2, 1], [3, 2], [4, 0], [-1, 1], [6, 0]], "y").ordonne())
            Polynome([[3, 2], [2, 1], [-1, 1], [4, 0], [6, 0]], "y", 0)

        :param type: Polynome
        :rtype: Polynome
        """
#         if len(self) == 1: return self
        m1 = self.monomes
        m1 = sorted(m1, key=lambda x: (-x[1]))
        return Polynome(m1, self.var, self.details)

    def nreduction(self, bdirecte=False):
        """**nreduction**\ (*object*)

        Retourne une étape de la réduction d'un polynome selon la valeur de *detail*
        Si bdirecte est True, alors donne la version réduite sans aucun détail de calcul

        :param type: Polynome
        :param bdirecte: faut-il donner directement le résultat de la réduction, sans étape de calcul ?
        :param type: boolean
        :rtype: Polynome
        """
        def _reduit_monome(monome, bdirecte):
            if isinstance(monome[0], (str, Fraction, SquareRoot)):
            # une réduction peut-être à faire ici
                if isinstance(monome[0], Fraction):
                    monome[0] = repr(monome[0].traitement(True))
                    res = []
                elif isinstance(monome[0], SquareRoot):
                    monome[0] = repr(monome[0].simplifie())
                    res = []
                else:
                    splitted = Priorites3.splitting(monome[0])
                    if len(splitted) > 1:
                        res = Priorites3.effectue_calcul(splitted)
                        if len(res) == 1:  # Le calcul s'effectue en une seule étape
                            monome[0] = res[0]
                if len(res) > 1:
                # Il y a un calcul à effectuer en plusieurs étapes
                    if bdirecte:  # on ne détaille pas
                        monome[0] = eval(Priorites3.priorites(monome[0])[-1][0])
                    else:
                        bdec = True
                        # *bdec* est un booléen qui est True si le calcul ne contient que des décimaux
                        for nb in res:
                            if nb[0] not in "+-*/0123456789(":
                                bdec = False
                                break

                        if bdec:  # on calcule en une étape 3+4-5
                            monome[0] = eval(Priorites3.priorites(monome[0])[-1][0])
                        else:
                            monome[0] = ''.join(res)  # on calcule étape par étape
                else:
                    monome[0] = eval(monome[0])  # le texte n'était pas nécessaire, c'est un nombre ou la fraction était irréductible
            return monome


        if not self.reductible(): return self
        else:
            if self.ordonnable() and self.details > 0 and not bdirecte:
                return self.ordonne()
            if not self.ordonne().reductible():
                # le polynome est seulement ordonnable
                return self.ordonne()
            # le polynome est donc réductible :
            factorisation, bfact = [], False
            # *factorisation* est une liste de monomes, qui va contenir les monomes du polynome en cours
            # de réduction, en stockant au format texte la somme des coefficients des monomes de même degré
            # *bfact* est un booléen qui est vrai si on doit effectuer une factorisation pour réduire le
            # polynome
            for m in self.ordonne().monomes:
                if factorisation and  m[1] == factorisation[-1][1]:
                    # deux monomes de même degré
                    bfact = True
                    if not isinstance(factorisation[-1][0], str): factorisation[-1][0] = repr(factorisation[-1][0])
                    if isinstance(m[0], str):
                        # On écrit la somme des deux coefficients au format str
                        factorisation[-1][0] += '+' + m[0]
                    elif m[0] > 0:
                        factorisation[-1][0] += '+%r' % m[0]
                        # On écrit la somme des deux coefficients au format str
                    elif m[0] < 0:
                        if isinstance(m[0], Fraction) and isinstance(m[0].d, str):
                            # On écrit la somme des deux coefficients au format str
                            factorisation[-1][0] += '+%r' % m[0]
                        else:
                            # On écrit la différence des deux coefficients au format str, '2-4'
                            # plutôt que '2+(-4)'
                            factorisation[-1][0] += '-%r' % -m[0]
                else:
                    # on ajoute un nouveau monome de degré inférieur au précédent
                    factorisation.append([m[0], m[1]])
            if bfact and self.details > 1 and not bdirecte:
                    # On retorune le polynome avec les factorisations mises en évidence
                    # Cas du monome de degré 0
                    if factorisation[-1][1] == 0 and isinstance(factorisation[-1][0], str):
                        factorisation[-1] = _reduit_monome(factorisation[-1], bdirecte)
                    return Polynome(factorisation, self.var, details=self.details)
            # on réduit le polynome
            for i in range(len(factorisation)):
                factorisation[i] = _reduit_monome(factorisation[i], bdirecte)
            return Polynome(factorisation, self.var, details=self.details)
            # Les monomes nuls sont retirés par la classe Polynome

    def _convert_other(self, other):
        """Convertit other en Polynome

        Utilisée dans les fonctions d'opérations entre Polynomes :add, sub, mul, ...
        """
        if isinstance(other, Polynome):
            if self.var == other.var:
                return other
            else:
                raise ValueError(u'Pyromaths ne sait manipuler que deux polynômes de même variable : %s et %s' % (self.var, other.var))
        if isinstance(other, (float, int, Fraction)):
            return Polynome([[other, 0]], self.var, self.details)
        return NotImplemented

def factoriser(calcul):
    """Factorise la somme calcul. Cette procédure n'est ni exaustive, ni
    robuste. Elle permet actuellement de résoudre les exercices proposés en 3e.

    :param calcul: Expression à factoriser
    :type calcul: str

    >>> from pyromaths.classes.PolynomesCollege import factoriser
    >>> factoriser("Polynome('4x^2+12x+9')")
    'Polynome([[2.0, 1]], "x", 0)**2+2*Polynome([[2.0, 1]], "x", 0)*Polynome([[3.0, 0]], "x", 0)+Polynome([[3.0, 0]], "x", 0)**2'
    >>> factoriser("Polynome('-4x^2+12x-9')")
    '-(Polynome([[2.0, 1]], "x", 0)**2-2*Polynome([[2.0, 1]], "x", 0)*Polynome([[3.0, 0]], "x", 0)+Polynome([[3.0, 0]], "x", 0)**2)'
    >>> factoriser("Polynome('4x^2-9')")
    'Polynome([[SquareRoot([[1, 4]]), 1]], "x", 0)**2-Polynome([[SquareRoot([[1, 9]]), 0]], "x", 0)**2'
    >>> factoriser("Polynome('3x^2-5')")
    'Polynome([[SquareRoot([[1, 3]]), 1]], "x", 0)**2-Polynome([[SquareRoot([[1, 5]]), 0]], "x", 0)**2'
    >>> factoriser("Polynome('4x^2')")
    'Polynome([[2.0, 1]], "x", 0)**2'
    >>> factoriser("Polynome('4x')")
    >>> factoriser("Polynome('x+1')*Polynome('x+2')+Polynome('x+1')*Polynome('x+3')")
    'Polynome([[1, 1], [1, 0]], "x", 0)*(Polynome([[1, 1], [2, 0]], "x", 0)+Polynome([[1, 1], [3, 0]], "x", 0))'
    >>> factoriser("Polynome('x+1')*Polynome('x+2')+Polynome('x+1')")
    'Polynome([[1, 1], [1, 0]], "x", 0)*Polynome([[1, 1], [2, 0]], "x", 0)+Polynome([[1, 1], [1, 0]], "x", 0)*1'
    >>> factoriser("Polynome('x+1')*Polynome('x+2')+Polynome('x+1')**2")
    'Polynome([[1, 1], [1, 0]], "x", 0)*Polynome([[1, 1], [2, 0]], "x", 0)+Polynome([[1, 1], [1, 0]], "x", 0)*Polynome([[1, 1], [1, 0]], "x", 0)'
    >>> factoriser("Polynome('x+1')**2-Polynome([[81, 0]], 'x', 3)")
    'Polynome([[1, 1], [1, 0]], "x", 0)**2-Polynome([[9.0, 0]], "x", 3)**2'

    :rtype: str
    """
    def id_rem(poly):
        """Vérifie si poly correspond au développement d'une identité
        remarquable ou est un carré.

        :param poly: polynome à évaluer
        :type poly: Polynome

        >>> from pyromaths.classes.PolynomesCollege import Polynome
        >>> from pyromaths.classes.PolynomesCollege.factoriser import id_rem
        >>> id_rem(Polynome('4x^2+12x+9'))
        "Polynome('2x')+2*Polynome('2x')*Polynome('3')+Polynome('3')**2"
        >>> id_rem(Polynome('-4x^2+12x-9'))
        "-(Polynome('2x')-2*Polynome('2x')*Polynome('3')+Polynome('3')**2)"
        >>> id_rem(Polynome('4x^2-9'))
        "(Polynome('2x')+Polynome('3'))*(Polynome('2x')-Polynome('3'))"
        >>> id_rem(Polynome('4x^2'))
        "Polynome('2x')**2"
        >>> id_rem(Polynome('4x'))
        None

        :rtype: str ou None
        """
        from math import sqrt
        from pyromaths.classes.SquareRoot import SquareRoot
        poly = poly.ordonne()
        if len(poly) == 1:
            if poly[0][1] % 2 == 0 and isinstance(poly[0][0], int) and \
              Arithmetique.carrerise(poly[0][0]) == 1:
                return "%r**2" % (Polynome([[sqrt(abs(poly[0][0])), poly[0][1] // 2]], poly.var, poly.details))
        elif len(poly) == 2:
            # if isinstance(SquareRoot([1, abs(poly[0][0])]).simplifie(),str):

            a = repr(Polynome([[SquareRoot([1, abs(poly[0][0])]), poly[0][1] // 2]], poly.var, poly.details))
            b = repr(Polynome([[SquareRoot([1, abs(poly[1][0])]), poly[1][1] // 2]], poly.var, poly.details))
            if not poly[0][1] % 2 and not poly[1][1] % 2:
                # Les deux exposants sont des carrés
                if poly[0][0] * poly[1][0] < 0:
                    # 3ème identité remarquable
                    if poly[0][0] < 0: return '%s**2-%s**2' % (b, a)
                    else: return '%s**2-%s**2' % (a, b)
        elif len(poly) == 3:
            a = repr(Polynome([[sqrt(abs(poly[0][0])), poly[0][1] // 2]], poly.var, poly.details))
            b = repr(Polynome([[sqrt(abs(poly[2][0])), poly[2][1] // 2]], poly.var, poly.details))
            if not poly[0][1] % 2 and not poly[2][1] % 2 and (poly[0][1] // 2 + poly[2][1] // 2 == poly[1][1]):
                # Ça ressemble à l'une des deux premières indentités remarquables
                if poly[0][0] * poly[2][0] > 0 and poly[0][0] * poly[1][0] > 0 and \
                      4 * poly[0][0] * poly[2][0] == poly[1][0] ** 2:
                    # 1ère identité remarquable, au signe près
                    if poly[0][0] > 0:
                        return a + '**2+2*' + a + '*' + b + '+' + b + '**2'
                    else:
                        return '-(' + a + '**2+2*' + a + '*' + b + '+' + b + '**2)'
                if poly[0][0] * poly[2][0] > 0 and poly[0][0] * poly[1][0] < 0 and \
                      4 * poly[0][0] * poly[2][0] == poly[1][0] ** 2:
                    if poly[0][0] > 0:
                        return a + '**2-2*' + a + '*' + b + '+' + b + '**2'
                    else:
                        return '-(' + a + '**2-2*' + a + '*' + b + '+' + b + '**2)'
        return None

    def EstFactorise(lcalcul):
        """indique si une liste de calculs est factorisée

        :param lcalcul: liste de calculs
        :type lcalcul: list

        >>> from pyromaths.classes.PolynomesCollege.factoriser import EstFactorise
        >>> EstFactorise(["Polynome('3x+4')", "*", "5"])
        True
        >>> EstFactorise(["Polynome('3x+4')", "*", "5", "+", "Polynome('2x+7')"])
        False

        :rtype: boolean
        """
        # Si le premier symbole est '+' ou '-', alors cela n'influe pas sur somme / produit
        dummy = [lcalcul[i] for i in range(len(lcalcul))]
        if dummy[0] == '+' or dummy[0] == '-': dummy = dummy[1:]
        par = Priorites3.recherche_parentheses(dummy)
        if par:
            if par[0] == 0 and par[1] == len(dummy):
                return EstFactorise(dummy[1:-1])
            while par:
                del dummy[par[0]:par[1]]
                par = Priorites3.recherche_parentheses(dummy)

            if '+' in dummy[1:] or '-' in dummy[1:]:
                return False
        else:
            if '+' in dummy or '-' in dummy:
                return False
        return True

    from pyromaths.outils import Arithmetique

    lcalcul = Priorites3.splitting(calcul)
    if len(lcalcul) == 1:
        # On tente une identité remarquable, un facteur commun ou une méthode Lycée (à faire plus tard)
        if 'Polynome(' == lcalcul[0][:9]:
            poly = eval(lcalcul[0])
            id_rem = id_rem(poly)
            if id_rem is None and len(poly) > 1:
                # Cherchons un facteur commun
                facteur = Arithmetique.pgcd([m[0] for m in poly])
                exp = min([m[1] for m in poly])
                if abs(facteur) != 1 or exp != 0:
                    k = Polynome([[facteur, exp]], poly.var, poly.details)
                    a = poly // k
                    return "*".join([repr(k), repr(a)])
                return None
            else:
                return id_rem
        else:
            raise ValueError(u'%s n\'est pas un Polynome et ne peut être factorisé')
    else:
        # C'est soit une somme de polynomes qu'il faut factoriser, soit déjà un produit
        if EstFactorise(lcalcul):
            # Il faut réduire les sommes
            poly = Priorites3.effectue_calcul(lcalcul)
            if EstFactorise(poly): return "".join(poly)
            else: return None
        else:
            # On cherche un facteur commun ou une identité remarquable
            nbpar = 0
            ltermes = [[]]
            if lcalcul[0] in '+-':
                lsgn = [lcalcul[0]]
            else:
                lsgn = ['']
                if lcalcul[0][:9] == 'Polynome(': ltermes = [[repr(eval(lcalcul[0]))]]
                else: ltermes = [[lcalcul[0]]]
            for el in lcalcul[1:]:
                if el in '+-' and nbpar == 0:
                    lsgn.append(el)
                    ltermes.append([])
                else:
                    if el == '(': nbpar += 1
                    elif el == ')': nbpar += -1
                    if el[:9] == 'Polynome(': ltermes[-1].append(repr(eval(el)))
                    else: ltermes[-1].append(el)

            if len(ltermes) >= 3 and lcalcul.count('**') > 1 and lcalcul.count('*') > 1 and lcalcul.count('2') > 2:
                # On peut être en présence de l'une des 2 premières identités remarquables
                exp1 = lcalcul.index('**')
                a = lcalcul[exp1 - 1]
                indexa = ltermes.index([a, '**', '2'])
                sgna = lsgn[indexa] or '+'
                exp2 = lcalcul[exp1 + 1:].index('**') + exp1 + 1
                b = lcalcul[exp2 - 1]
                indexb = ltermes.index([b, '**', '2'])
                sgnb = lsgn[indexb] or '+'

                for i in range(len(ltermes)):
                    if len(ltermes[i]) == 5 and ltermes[i].count('2') > 0 and \
                      ltermes[i].count('*') == 2 and ltermes[i].count(a) and \
                      ltermes[i].count(b):
                        indexp = i
                        sgnp = lsgn[i] or '+'

                if sgna == sgnb and sgna != sgnp:
                    poly = '(%s-%s)**2' % (a, b)
                elif sgna == sgnb and sgna == sgnp:
                    poly = '(%s+%s)**2' % (a, b)
                else:
                    # Pas d'identité remarquable
                    poly = None
                if poly:
                    if sgna == '-': poly = '-%s' % poly
                    lindex = [indexa, indexb, indexp]
                    lindex.sort()
                    lindex.reverse()
                    for i in lindex:
                        lsgn.pop(i)
                        ltermes.pop(i)

                    ltermes.insert(min(len(ltermes), indexa, indexb, indexp), Priorites3.splitting(poly))
                    res = []
                    for i in range(len(ltermes)):
                        # TODO: IndexError: list index out of range

                        if lsgn and lsgn[i]: res.append(lsgn[i])
                        res.extend(ltermes[i])
                    return "".join(res)

            if (('+' in lsgn or '' in lsgn) and '-' in lsgn) and calcul.count("**") == lcalcul.count("2") == 2 and len(ltermes) == 2:
                # 3ème identité remarquable
                # print ltermes[0][0], ltermes[1][0]
                a, b = eval(ltermes[0][0]), eval(ltermes[1][0])
                if lsgn[0] == '-': return '(%r+%r)*(%r-%r)' % (b, a, b, a)
                else: return '(%r+%r)*(%r-%r)' % (a, b, a, b)

            for k in ltermes[0]:
                commun = False
                if k[:9] == 'Polynome(':
                    commun = False
                    bdetail = False
                    pos = ltermes[0].index(k)
                    if len(ltermes[0]) == 1: bdetail = True
                    elif pos < len(ltermes[0]) - 1 and ltermes[0][pos + 1] == "**": bdetail = True
                    for i in range(len(ltermes) - 1):
                        if ltermes[i + 1].count(k):
                            commun = True
                            pos = ltermes[i + 1].index(k)
                            if len(ltermes[i + 1]) == 1: bdetail = True
                            elif pos < len(ltermes[i + 1]) - 1 and ltermes[i + 1][pos + 1] == "**": bdetail = True
                        else:
                            commun = False
                            break
                if commun: break
            if commun:
                for i in range(len(ltermes)):
                    pos = ltermes[i].index(k)
                    if len(ltermes[i]) == 1:
                        ltermes[i].extend(["*", "1"])
                    elif pos < len(ltermes[i]) - 1 and ltermes[i][pos + 1] == "**" and ltermes[i][pos + 2] == "2":
                        ltermes[i][pos + 1:pos + 3] = ["*", ltermes[i][pos]]
                    elif pos < len(ltermes[i]) - 1 and ltermes[i][pos + 1] == "**":
                        ltermes[i][pos + 2] == str(eval(ltermes[i][pos + 2]) - 1)
                    elif not bdetail:
                        del ltermes[i][pos]
                        if pos: del ltermes[i][pos - 1]
                        # supprime le symbole *
                        else: del ltermes[i][pos]
                a = []
                for i in range(len(ltermes)):
                    if lsgn[i]:a.append(lsgn[i])
                    a.extend(ltermes[i])
                if bdetail:
                    # On affiche le facteur commun de façon claire
                    return "".join(a)
                else:
                    # On effectuer la factorisation par le facteur commun
                    return "%s*(%s)" % (k, "".join(a))
            else:
                bfact = False
                for i in range(len(lcalcul)):
                    if 'Polynome(' == lcalcul[i][:9]:
                        poly = id_rem(eval(lcalcul[i]))
                        if poly:
                            bfact = True
                            lcalcul[i:i + 1] = Priorites3.splitting(poly)
                        else:
                            lcalcul[i] = repr(eval(lcalcul[i]))
                if bfact:
                    return "".join(lcalcul)
                else:
                    return None

def choix_points(min=-4, max=4, nb=5):
    """**choix_points**\ ()

    Renvoie un tuple contenant nb coordonnées sous forme de tuple telles que
    les abscisses et ordonnées sont distinctes, comprises entre min et max,
    une abscisse n'est jamais égale à une ordonnée et la coordonnée (b, a) n'est
    pas listée si la coordonnée (a, b) existe.

    >>> from pyromaths.ex.troisiemes import notion_de_fonction
    >>> notion_de_fonction.choix_points()  # doctest: +SKIP
    ((-4, -2), (-2, 0), (0, 4), (2, -4), (4, 2))

    :rtype: tuple

    """
    if nb > max - min + 1:
        raise ValueError('On demande trop de points vu le min et le max')
    abscisse = [i for i in range(min, max + 1)]
    for dummy in range(max - min - nb):
        del abscisse[randrange(len(abscisse))]
    refaire = True
    while refaire:
        ordonnee = [abscisse[i] for i in range(nb)]
        shuffle(ordonnee)
        refaire = False
        for i in range(nb):
            if abscisse[i] == ordonnee[i] or abscisse.index(ordonnee[i]) == ordonnee.index(abscisse[i]):
                refaire = True
                break
    return tuple([(abscisse[i], ordonnee[i]) for i in range(nb)])

def Lagrange(points):
    """**Lagrange**\ (*points*)
    Renvoie le polynôme d'interpolation de Lagrange pour les points de coordonnées *points*

    Est prévue pour être utilisé avec :py:func:`choix_points`

    Associé à  :py:func:`pyromaths.outils.Priorites3.priorites`, permet d'obtenir sa version réduite.

    Associé à  :py:func:`pyromaths.outils.Priorites3.plotify`, permet d'obtenir sa version utilisable avec psplot.

    >>> from pyromaths.ex.troisiemes import notion_de_fonction
    >>> p = notion_de_fonction.Lagrange(((-4, -2), (-2, 0), (0,4), (2,-4), (4,2)))

    >>> from pyromaths.outils import Priorites3
    >>> Priorites3.priorites(p)[-1]
    ['Polynome([[Fraction(5, 48), 4], [Fraction(1, 8), 3], [Fraction(-23, 12), 2], [Fraction(-3, 2), 1], [Fraction(4, 1), 0]], "x", 0)']
    >>> Priorites3.plotify(Priorites3.priorites(p)[-1])
    5/48*x^4+1/8*x^3-23/12*x^2-3/2*x^1+4/1
    """
    PIL = []
    for i in range(len(points)):
        if points[i][1]:
            PIL.append('Fraction(%s, ' % points[i][1])
            produit = []
            for j in  range(len(points)):
                if j != i: produit.append(repr(points[i][0] - points[j][0]))
            produit = repr(eval("*".join(produit)))
            PIL[i] += produit + ")*"
            produit = []
            for j in  range(len(points)):
                if j != i and points[j][0] > 0: produit.append("Polynome(\"x-%s\", details = 0)" % points[j][0])
                elif j != i and points[j][0] < 0: produit.append("Polynome(\"x+%s\", details = 0)" % -points[j][0])
                elif j != i and points[j][0] == 0: produit.append("Polynome(\"x\", details = 0)")
            produit = "*".join(produit)
            PIL[i] += produit
        else:
            PIL.append('0')
    return "+".join(PIL)

